#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <bits/nan.h>

#include <tcl.h>

typedef char bool;

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

/*
 *        Prototypes
 */
int Tcl_DACCmd(ClientData dummy, Tcl_Interp * interp, int argc, char **argv);
int Tcl_ADCCmd(ClientData dummy, Tcl_Interp * interp, int argc, char **argv);
int Tcl_IEEECmd(ClientData dummy, Tcl_Interp * interp, int argc, char **argv);
int Tcl_NwaCmd(ClientData dummy, Tcl_Interp * interp, int argc, char **argv);
int Tcl_TekCmd(ClientData dummy, Tcl_Interp * interp, int argc, char **argv);
int Tcl_DoutCmd(ClientData dummy, Tcl_Interp * interp, int argc, char **argv);
int Tcl_ParportCmd(ClientData dummy, Tcl_Interp * interp, int argc, char **argv);

/*
 * Globale Variablen
 */

char units[]="afpnum kMGT"; 

/*
 * Unterprogramme
 */

double ValIn(char *input, char *unit)
{
	char NOT_FOUND=TRUE;
	double value;
	int count, i;

	unit[0]=0;
	count = sscanf(input, "%lf%10s", &value, unit);

	if (count<1 || count >2) {
		unit[0]=0;
		return NAN;
	}
	
	if (count == 1) {
		return value;
	} else {
		for (i=0; i<=10; i++) {
			if (units[i]==unit[0]) {
				NOT_FOUND=FALSE;
				value *= pow(10,(i-6)*3);
			}
		}
		if (NOT_FOUND) {
			return value;
		} else {
			for (i=0; unit[i]!=0; i++)
				unit[i]=unit[i+1];
			return value;
		}
	}
}

int Tcl_ValIn(ClientData dummy, Tcl_Interp * interp, int argc, char **argv)
{
	double value;
	char unit[50];
	char result[100];

	if (argc != 2) return TCL_ERROR;

	value = ValIn(argv[1], unit);
	
	if (unit[0] !=0) {
		sprintf(result, "%g %s", value, unit);
	} else {
		sprintf(result, "%g", value);
	}
	
	Tcl_AppendResult(interp, result, (char *) NULL);
	return TCL_OK;
}

void ValOut(double value, char *result, int maxdigits)
{
	double sign=1.0;
	int index, leftdigits, rightdigits;

	if (value==0.0) {
		sprintf(result, "0.0");
	} else {
		if (value<0) {
			value *= -1.0;
			sign = -1.0;
		}
		
		index = floor(log10(value)/3.0);
	
		if ((index < -6) || (index > 4)) {
			sprintf(result, "%7.6g", sign*value);
		} else {
			value = value/pow(10,index*3);
			leftdigits = ceil(log10(value));
			
			// falls <maxdigits> > 0 : Maximale Stellenanzahl
			// falls <maxdigits> < 0 : Maximale Nachkommastellenanzahl
			
			if (maxdigits>0) {
				maxdigits -= leftdigits;
			} else {
				maxdigits = -maxdigits;
			}
			
			for (rightdigits=0; rightdigits<maxdigits; rightdigits++) {
				double shiftvalue = value*pow(10,rightdigits);

				if (floor(shiftvalue) == shiftvalue) {
					break;
				}
			}
			if (index!=0) {
				sprintf(result, "%.*f%c",rightdigits , sign*value, units[index+6]);
			} else {
				sprintf(result, "%.*f",rightdigits , sign*value);
			}
		}
	}
}

int Tcl_ValOut(ClientData dummy, Tcl_Interp * interp, int argc, char **argv)
{
	double value;
	char result[100];
	int maxdigits;

	if (argc < 2 || argc >3) return TCL_ERROR;
	
	// Zuerst den Wert einlesen
	if (sscanf(argv[1], "%lf", &value) != 1) return TCL_ERROR;
	
	// Dann, wenn vorhanden die Anzahl der auszugebenden Stellen angeben
	if (argc == 3) {
		if (sscanf(argv[2], "%d", &maxdigits) != 1) return TCL_ERROR;
	} else {
		maxdigits = 4;
	}
	ValOut(value, result, maxdigits);
	
	Tcl_AppendResult(interp, result, (char *) NULL);
	return TCL_OK;
}

char * Exp_Config_Dir()
{
	static char ConfigDir[100]="";
	
	if (ConfigDir[0]==0) {
		FILE *PathFile = fopen("/etc/Exp_Config_Dir", "r");		/* Pfaddatei oeffnen */
	
		if (PathFile != NULL) {
			fscanf(PathFile, "%s\n", ConfigDir);
			fclose(PathFile);
		}
	}
	
	if (ConfigDir[0]==0)
		return ".";
	else
		return ConfigDir;
}

int Tcl_Exp_Config_Dir(ClientData dummy, Tcl_Interp * interp, int argc, char **argv)
{
	Tcl_AppendResult(interp, Exp_Config_Dir(), (char *) NULL);
	return TCL_OK;
}


/*
 *        Init-Function
 */

int Tclmod_Init(Tcl_Interp * interp)
{
	Tcl_CreateCommand(interp, "ValIn", Tcl_ValIn, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(interp, "ValOut", Tcl_ValOut, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(interp, "Exp_Config_Dir", Tcl_Exp_Config_Dir, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(interp, "ieee", Tcl_IEEECmd, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(interp, "nwa", Tcl_NwaCmd, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(interp, "tek", Tcl_TekCmd, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(interp, "adc", Tcl_ADCCmd, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(interp, "dac", Tcl_DACCmd, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(interp, "dout", Tcl_DoutCmd, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(interp, "pout", Tcl_ParportCmd, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	return TCL_OK;
}
