/* 
 * tclopto_DAC.c --
 *
 *  This file contains the top-level command routines for the
 *  the Kryo-spezific built-in commands 
 *
 * Copyright (c) 1987-1993 The Regents of the University of California.
 * Copyright (c) 1994-1995 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#define AVS_DEVICE "/dev/opto0"

#define TIMEOUT_AVS 2000
#define AVS_RETRY 20			/* Retrys, if AVS busy ... (alle 250msec) */
#define AVS_TIMEOUT 0			/* Return after 20 retries ...  */
#define AVS_INSIST  1			/* Be very insistive ...  */
#define AVS_TEST    2			/* Test, for retries ...  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/times.h>

#include <tcl.h>

#include "optolib.h"

/* Prototypen */
void ValOut(double value, char *result);

/********************************************************************/
/* Ist die AVS an Kanalnr kanr angeschlossen so knnen durch Angabe */
/* der Latchnr (0-2) die Internen Latches der Brcke gelesen werden */
/********************************************************************/

int readavs(int kanr, int latchnr, int *wert)
{
	/* resetport(); *//* ist notwendig, um evt. falsches Empfangsflag zu lschen */
	/* Befehlbermittlung und Antwort holen TIMEOUT_AVS ms timeout */
	*wert = optosendempf(kanr, latchnr | 0x80, 0, TIMEOUT_AVS);
	if (*wert < 0)
		return -2;				/* return *wert; */
	return 0;
};

/********************************************************************/
/* Ausfhrungszeit ist immer < 5ms mit Standard Atari 1040.        */
/* avswert liefert in wert den Zahlenwert der Anzeige, in range den */
/* Messbereich. Der Messwert in Ohm = wert * 10**(range-5).         */
/* Range besitzt die Werte 1-7, Excitation 1-6 == (10uV-3mF).       */
/* In exci wird die Excitation zurckgeliefert.                    */
/* Das Funktionsergebnis  0 bedeutet kein Fehler aufgetreten,       */
/* -1 bedeutet Timeout beim Befehlsenden, -2 Timeout beim empfangen, */
/* also AVS antwortet nicht und -3 wenn AVS gerade nicht bereit.    */
/* Ist rangein > 0 und ungleich dem eingestelltem Range an der      */
/* AVS, so wird die Differenz um eins errniedrigt. Gleiches gilt fuer */
/* exciin. Ist rangein = 0 findet keine Rangevernderung statt.     */
/* (gilt auch fr exciin). Ist rangein = -1 wird die Overrange-     */
/* automatik eingeschalten, ist rangein =-2 wird zustzlich eine    */
/* Rangekorrektur nach unten ausgehrt, wenn der Messwert < 1900 ist */
/* bei jeder Korrektur von Range u/o Excitaion wird kein Mewert    */
/* gelesen (nicht sinnvoll) der Returnwert ist dann -4.             */
/* *range und *exci enthalten immer die aktuellen Werte solange     */
/* der Returnwert nicht -1 oder -2 betrgt.                         */
/********************************************************************/

int avswert(int *wert, int *range, int *exci, int *mux, int rangein, int exciin, int muxin)
{
	int L0, L1, L2, L3, exra = 0xC0, muxch = 0x0, fehlnr = 0, kanr;
	int pol = -1, over = 0;

	kanr = addsearch("AVS");	/* Default-Optoadd suchen */

	fehlnr = readavs(kanr, 3, &L3);		/* latch 3 lesen */
	if (fehlnr != 0)
		goto ende;
	*mux = L3;

	if (muxin >= 0) {			/* Excitatioqn berprfen */
		if (*mux < muxin)		/* Mux eins aufwrts */
			muxch |= 0x100;
		else if (*mux > muxin)	/* Mux eins abwrts */
			muxch |= 0x200;
	};

	fehlnr = readavs(kanr, 0, &L0);		/* latch 0 lesen */
	if (fehlnr != 0)
		goto ende;

	fehlnr = readavs(kanr, 1, &L1);		/* Latch 1 lesen */
	if (fehlnr != 0)
		goto ende;

	fehlnr = readavs(kanr, 2, &L2);		/* Latch 2 lesen */
	if (fehlnr != 0)
		goto ende;

	*exci = (L0 & 0x1C0) / 0x40;
	*range = (L0 & 0x38) / 0x8;
	if (L0 & 0x4)
		pol = 1;
	if (L0 & 0x2)
		over = 1;

	if (rangein > 0) {			/* vorgegebenen Range einstellen */
		if (*range < rangein)
			exra |= 0x8;		/* Range up */
		else if (*range > rangein)
			exra |= 0x4;		/* Range down */
	} else if ((rangein < 0) && (L0 & 0x2)) {	/* overrange */
		exra |= 0x8;			/* Range up */
	};

	if (exciin > 0) {			/* Excitation berprfen */
		if (*exci < exciin)		/* Excitation eins aufwrts */
			exra |= 0x20;
		else if (*exci > exciin)	/* Excitation eins abwrts */
			exra |= 0x10;
	};

	if (exra != 0xC0) {			/* Range und/oder Excieinstellung verndern */
		fehlnr = optosend(kanr, exra);
		if (fehlnr == 0) {
			avswert(wert, range, exci, mux, rangein, exciin, muxin);
		}
		goto ende;				/* auf alle Flle abbrechen, da Messwert noch nicht da */
	};

	if ((L0 & 0x200) == 0) {
		fehlnr = -3;
		goto ende;
	}							/* Busy ist aktiviert => Messwert sollte nicht gelesen werden */
	*wert = L0 & 1;				/* erste Stelle von Latch 0 */

	/* fehlnr = readavs(kanr,1,&L1); *//* Latch 1 lesen */
	if (fehlnr != 0)
		goto ende;
	*wert = 10 * (*wert) + (L1 & 0xf0) / 0x10;
	*wert = 10 * (*wert) + (L1 & 0xf);

	/* fehlnr = readavs(kanr,2,&L2); *//* Latch 2 lesen */
	if (fehlnr != 0)
		goto ende;
	*wert = 10 * (*wert) + (L2 & 0xf0) / 0x10;
	*wert = 10 * (*wert) + (L2 & 0xf);

	*wert = *wert * pol;

	if (*wert < 1900 && rangein == -2) {	/* Range um 1 ernidriegen */
		fehlnr = optosend(kanr, 0xC4);
		if (fehlnr == 0)
			*range -= 1;
	};

  ende:

	return fehlnr;
}

int avsread(int *wert, int *range, int *exci)
{
	int l0, fehlnr = 0, kanr;

	kanr = addsearch("AVS");	/* Pulseradd suchen */

	fehlnr = readavs(kanr, 0, &l0);		/* latch 0 lesen */
	if (fehlnr != 0) {
		printf("Kann nicht von AVS lesen!\n");
		goto ende;
	}
	*exci = (l0 & 0x1C0) / 0x40;
	*range = (l0 & 0x38) / 0x8;

	*wert = l0 & 1;				/* erste Stelle von Latch 0 */

	fehlnr = readavs(kanr, 1, &l0);		/* Latch 1 lesen */
	if (fehlnr != 0)
		goto ende;
	*wert = 10 * (*wert) + (l0 & 0xf0) / 0x10;
	*wert = 10 * (*wert) + (l0 & 0xf);

	fehlnr = readavs(kanr, 2, &l0);		/* Latch 2 lesen */
	if (fehlnr != 0)
		goto ende;
	*wert = 10 * (*wert) + (l0 & 0xf0) / 0x10;
	*wert = 10 * (*wert) + (l0 & 0xf);

  ende:
	return fehlnr;
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_AVSCmd --
 *
 *  This procedure is invoked to communicate with the AVS-Resistance-Bridge
 *      using the 
 *  See the user documentation for details on what it does.
 *
 * Results:
 *  A standard Tcl result.
 *
 * Side effects:
 *  See the user documentation.
 *
 *----------------------------------------------------------------------
 */

int Tcl_AVSCmd(ClientData dummy, Tcl_Interp * interp, int argc, char **argv)
{
	double AVS_Range[] = {0.0001, 0.001, 0.01, 0.1, 1, 10, 100};
	double AVS_Excitation[] = {10e-6, 30e-6, 100e-6, 300e-6, 1e-3, 3e-3};
	int err = 0;

	int argindex, i = 1;
	char argchar;

	int mode = AVS_TIMEOUT;

	/* Optionen parsen und aus dem Argumentfeld entfernen: */

	for (argindex = 1; argindex < argc;) {
		if (argv[argindex][0] == '-') {

			/* Hier die Optionen abfragen: */
			if (strcmp(argv[argindex], "-insist") == 0) {
				mode = AVS_INSIST;
			} else if (strcmp(argv[argindex], "-timeout") == 0) {
				mode = AVS_TIMEOUT;
			} else if (strcmp(argv[argindex], "-test") == 0) {
				mode = AVS_TEST;
			} else {
				Tcl_AppendResult(interp, "ERROR: unknown option '", argv[argindex], "'\n", (char *) NULL);
				return TCL_ERROR;
			}

			/* Zeiger auf Argumente verschieben: */
			if (argindex < argc) {
				for (i = argindex; i < argc; i++)
					argv[i] = argv[i + 1];
			}
			argc--;
		} else
			argindex++;
	}

	if (argc < 2)
		goto AVS_Help;

	/* Argumentindex wieder auf den Beginn der Argumente setzen */
	argindex = 1;

	argchar = argv[argindex][0];
	

/*** read ***/
	if ((argchar == 'r') && (strcmp(argv[argindex], "read") == 0)) {
		int wert, range, exci;
		char buffer[100];
		char result[100];
		
		if (argc != 2) {
			Tcl_AppendResult(interp, "wrong # args:\n", (char *) NULL);
			goto AVS_Help;
		}
		
		opto_open(AVS_DEVICE);
		avsread(&wert, &range, &exci);
		opto_close();

		/* Anzeigewert mit dem richtigen Messbereich multiplizieren */
		/* und formatieren */
		ValOut(wert*AVS_Range[range-1], result);
		
		sprintf(buffer, "%sOhm (r:%i, e:%i)\n", result, range, exci);
		Tcl_AppendResult(interp, &buffer, (char *) NULL);
		return TCL_OK;

/*** set ***/
	} else if ((argchar == 's') && (strcmp(argv[argindex], "set") == 0)) {
		int wert, range, exci, mux;
		int rangein=-1, exciin=-1, muxin=-1;
		char buffer[100];
		
		if ((argc < 3) || (argc > 5)) {
			Tcl_AppendResult(interp, "wrong # args:\n", (char *) NULL);
			goto AVS_Help;
		}
		argindex++;

		rangein = atoi(argv[argindex++]);
		if (argc >= 4) {
			exciin = atoi(argv[argindex++]);
			if (argc == 5)
				muxin = atoi(argv[argindex]);
		}
		if ((rangein < 1 || rangein > 7) && (rangein != -1)) {
			Tcl_AppendResult(interp, "wrong value for <range>\n", (char *) NULL);
			goto AVS_Help;
		}
		if ((exciin < 1 || exciin > 6) && (exciin != -1)) {
			Tcl_AppendResult(interp, "wrong value for <excitation>\n", (char *) NULL);
			goto AVS_Help;
		}
		if ((muxin < 0 || muxin > 7) && (muxin != -1)) {
			Tcl_AppendResult(interp, "wrong value for <mux-port>\n", (char *) NULL);
			goto AVS_Help;
		}
		opto_open(AVS_DEVICE);
		avswert(&wert, &range, &exci, &mux, rangein, exciin, muxin);
		opto_close();

		sprintf(buffer, "val:%i\trange:%i\texci:%i\n", wert, range, exci);
		Tcl_AppendResult(interp, &buffer, (char *) NULL);
		return TCL_OK;

	} else {
		Tcl_AppendResult(interp, "bad command \"", argv[i], "\"\n");
		goto AVS_Help;
	}

AVS_Error:
	switch (err) {
	case -1:
		Tcl_AppendResult(interp, "# Timeout while sending ... ", (char *) NULL);
		break;
	case -2:
		Tcl_AppendResult(interp, "# Timeout while reading ... ", (char *) NULL);
		break;
	case -3:
		Tcl_AppendResult(interp, "# AVS busy or disabled ... ", (char *) NULL);
		break;
	case -4:
		Tcl_AppendResult(interp, "# AVS reading invalid ... ", (char *) NULL);
		break;
	}
	return TCL_ERROR;

  AVS_Help:
	Tcl_AppendResult(interp, "USAGE:  avs [option] command [range] [excitation]\n\n",
					 "    option: -timeout (default), -insist or -test\n",
					 "   command: read \n",
					 "            set range [excitation [mux]]\n",
					 "                 range:     1...7 (2...2MOhm)\n",
					 "                 excitaton: 1...6\n",
					 "                       mux: 0 (front), 1...7", (char *) NULL);
	return TCL_ERROR;
}
