
/*
 *----------------------------------------------------------------------
 *
 * Tcl_DYNACmd --
 *
 *  Binary-Support for the Dynatrac Login 
 *      using the 
 *  See the user documentation for details on what it does.
 *
 * Results:
 *  A standard Tcl result.
 *
 * Side effects:
 *  See the user documentation.
 *
 *----------------------------------------------------------------------
 */

#define CONVERT_ASCII 1

	/* ARGSUSED */
int Tcl_DYNACmd(dummy, interp, argc, argv)
	 ClientData dummy;			/* Not used. */
	 Tcl_Interp *interp;		/* Current interpreter. */
	 int argc;					/* Number of arguments. */
	 char **argv;				/* Argument strings. */
{
	size_t length;
	int argc_ohne_option;
	int mode;

	int c, i = 1;

	FILE *out = NULL;
	char *in;
	signed long val = 0;
	int p = 0, k;
	char tab[2] =
	{'\t', '\n'};

#define DEVICE "DYNA"
#define DYNA_MAXDATA 256
#define DYNA_MAXLEN 2*3*DYNA_MAXDATA+2

	char GPIB_Buffer[DYNA_MAXLEN + 1];
	char out_Buffer[DYNA_MAXDATA * 18 + 1];
	int out_p = 0;
	int GPIB_unit;
	int GPIB_len;

	argc_ohne_option = argc;

	c = argv[i][0];
	if (c == '-') {
		length = strlen(argv[i]);
		if (strncmp(argv[i], "-acsii", length) == 0) {
			mode = CONVERT_ASCII;
		} else {
			Tcl_AppendResult(interp, "bad option \"", argv[i], "\"\n", (char *) NULL);
			goto Dyna_Help;
		}

		i++;
		argc_ohne_option--;
	}
	if (argc_ohne_option < 2) {
		Tcl_AppendResult(interp, "wrong # args:\n", (char *) NULL);
		goto Dyna_Help;
	}
	c = argv[i][0];
	length = strlen(argv[i]);
	if ((c == 'r') && (strncmp(argv[i], "read", length) == 0)) {
		if (argc_ohne_option == 3) {
			if ((out = fopen(argv[i + 1], "a")) == NULL) {
				Tcl_AppendResult(interp, "Can't open Datafile: \"", argv[i + 1], "\"\n", (char *) NULL);
				return TCL_ERROR;
			}
		} else if (argc_ohne_option != 2) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " [options] read [logfile]\"\n", (char *) NULL);
			goto Dyna_Help;
		}
		GPIB_unit = ibfind(DEVICE);
#ifdef DEBUG
		fprintf(stderr, "UNIT(%s)=%d\n", DEVICE, GPIB_unit);
#endif /* DEBUG */

		if (ibrd(GPIB_unit, GPIB_Buffer, DYNA_MAXLEN) & ERR) {
			Tcl_AppendResult(interp, "ERROR while reading DYNA\n", (char *) NULL);
			return TCL_ERROR;
		}
		GPIB_len = ibcnt;
#ifdef DEBUG
		fprintf(stderr, "Transfered %d Bytes\n", GPIB_len);
#endif /* DEBUG */

		in = &GPIB_Buffer[0];

		for (k = 0; k < (GPIB_len - 2); k += 3) {
			val = in[k] * 65535;
			if (in[k] & 0x80) {
				val |= 0xff000000;
			}
			val += in[k + 1] * 256;
			val += in[k + 2];

			if (out) {
#ifdef DEBUG
				fprintf(out, "%3d: %ld (0x%08x) %c", k, val, val, tab[p]);
#else /* DEBUG */
				fprintf(out, "%ld%c", val, tab[p]);
#endif /* DEBUG */
			} else {
				out_p += sprintf(&out_Buffer[out_p], "% 7ld%c", val, tab[p]);
#ifdef DEBUG
				fprintf(stderr, "%3d: %ld (0x%08x) %c", out_p, val, val, tab[p]);
#endif /* DEBUG */
			}

			if (p) {
				p = 0;
			} else {
				p = 1;
			}
		}

		if (out) {
			fclose(out);
			sprintf(interp->result, "%d %d %d 0x%04x", GPIB_len / 6, ibcnt, iberr, ibsta);
		} else {
			interp->result = out_Buffer;
		}
		return TCL_OK;
	} else {
		Tcl_AppendResult(interp, "bad command \"", argv[i], "\"", (char *) NULL);
		goto Dyna_Help;
	}

  Dyna_Help:
	Tcl_AppendResult(interp, "USAGE: dyna [option] command \n",
					 "    option:  -ascii   : Convert to ASCII  (default)\n",
					 "    command: read\n"
					 "       read [logfile] : read and append to logfile\n",
					 (char *) NULL);
	return TCL_ERROR;
}
