
/* 
 * tclOPTO.c --
 *
 *  This file contains the top-level command routines for the
 *  the Kryo-spezific built-in commands 
 *
 * Copyright (c) 1987-1993 The Regents of the University of California.
 * Copyright (c) 1994-1995 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "optolib.h"

#include <tcl.h>

#define IEEE_MAXLEN 64000

#define IEEE_DEVICE "/dev/opto0"

/*
 *----------------------------------------------------------------------
 *
 * Tcl_IEEECmd --
 *
 *  This procedure is invoked to communicate with the IEEE-Bus-System
 *      using the optolib.c - Functions 
 *  See the user documentation for details on what it does.
 *
 * Results:
 *  A standard Tcl result.
 *
 * Side effects:
 *  See the user documentation.
 *
 *----------------------------------------------------------------------
 */

int Tcl_IEEECmd(ClientData dummy, Tcl_Interp * interp, int argc, char **argv)
{
	size_t arglength;
	int argindex;
	char argchar = 0;

	int ret, mode = 0x0000;

	char IEEE_Buffer[IEEE_MAXLEN + 1];
	int IEEE_Int;

	if (argc < 2)
		goto IEEE_Help;

	/* Optionen parsen und aus dem Argumentfeld entfernen: */

	for (argindex = 1; argindex < argc;) {
		if (argv[argindex][0] == '-') {

			/* Hier die Optionen abfragen: */
			arglength = strlen(argv[argindex]);

			if (strncmp(argv[argindex], "-EOI", arglength) == 0) {
				mode = 0x0000;
			} else if (strncmp(argv[argindex], "-CR", arglength) == 0) {
				mode = 0x000d;
			} else if (strncmp(argv[argindex], "-CRLF", arglength) == 0) {
				mode = 0x0d0a;
			} else {
				Tcl_AppendResult(interp, "ERROR: unknown option '", argv[argindex], "'\n", (char *) NULL);
				return TCL_ERROR;
			}

			/* Zeiger auf Argumente verschieben: */
			if (argindex < argc) {
				int i;
				for (i = argindex; i < argc; i++)
					argv[i] = argv[i + 1];
			}
			argc--;
		} else
			argindex++;
	}

	/* Argumentindex wieder auf den Beginn der Argumente setzen */
	argindex = 1;

	/* Nun die Argumente parsen: */

	if (argc < 2) {
		Tcl_AppendResult(interp, "wrong # args:\n", (char *) NULL);
		goto IEEE_Help;
	}
	argchar = argv[argindex][0];
	arglength = strlen(argv[argindex]);

/*** alloc ***/
	if ((argchar == 'a') && (strncmp(argv[argindex], "alloc", arglength) == 0)) {
		if (argc != 2) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " alloc\"", (char *) NULL);
			return TCL_ERROR;
		}
		opto_open(IEEE_DEVICE);
		return TCL_OK;

/*** dealloc ***/
	} else if ((argchar == 'd') && (strncmp(argv[argindex], "dealloc", arglength) == 0)) {
		if (argc != 2) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " dealloc\"", (char *) NULL);
			return TCL_ERROR;
		}
		opto_close();
		return TCL_OK;

/*** gtl ***/
	} else if ((argchar == 'g') && (strncmp(argv[argindex], "gtl", arglength) == 0)) {
		if (argc < 3) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " gtl IEEE_address ?IEEE_address? ...\"", (char *) NULL);
			return TCL_ERROR;
		}
		argindex++;

		opto_open(IEEE_DEVICE);
		for (; argindex < argc; argindex++) {
			ret = gtl(argv[argindex]);
			if (ret < 0) {
				opto_close();
				sprintf(interp->result, "%d", ret);
				Tcl_AppendResult(interp, "# Error sending to IEEE ...", argv[0],
								 argv[1], argv[argindex], (char *) NULL);
				return TCL_ERROR;
			}
		}
		opto_close();
		return TCL_OK;

/*** read ***/
	} else if ((argchar == 'r') && (strncmp(argv[argindex], "read", arglength) == 0)) {
		if (argc != 3) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " read IEEE_address\"\n", (char *) NULL);
			goto IEEE_Help;
		}
		argindex++;

		opto_open(IEEE_DEVICE);
		ret = i3e_input(argv[argindex], IEEE_Buffer, mode);
		opto_close();

		if (ret >= 0) {
			IEEE_Buffer[ret] = 0;	/* gesendete Zeichenkette terminieren */
			Tcl_AppendResult(interp, IEEE_Buffer, (char *) NULL);
			return TCL_OK;
		} else {
			interp->result[0] = 0;
			Tcl_AppendResult(interp, "# Error reading from IEEE ...", argv[0],
							 argv[argindex - 1], argv[argindex], (char *) NULL);
			return TCL_ERROR;
		}

/*** reset ***/
	} else if ((argchar == 'r') && (strncmp(argv[argindex], "reset", arglength) == 0)) {
		if (argc < 3) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " reset IEEE_address ?IEEE_address? ...\"\n", (char *) NULL);
			goto IEEE_Help;
		}
		argindex++;

		opto_open(IEEE_DEVICE);
		for (; argindex < argc; argindex++) {
			power_on(addsearch(argv[argindex]));
		}
		opto_close();
		return TCL_OK;

/*** send ***/
	} else if ((argchar == 's') && (strncmp(argv[argindex], "send", arglength) == 0)) {
		if (argc != 4) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " send IEEE_address IEEE_string\"\n", (char *) NULL);
			goto IEEE_Help;
		}
		argindex++;

		opto_open(IEEE_DEVICE);
		ret = i3e_output(argv[argindex], argv[argindex + 1], strlen(argv[argindex + 1]));
		opto_close();

		if (ret < 0) {
			sprintf(interp->result, "%d", ret);
			Tcl_AppendResult(interp, "# Error sending to IEEE ...", argv[0],
							 argv[1], argv[argindex], argv[argindex + 1], (char *) NULL);
			return TCL_ERROR;
		}
		return TCL_OK;

/*** sdc ***/
	} else if ((argchar == 's') && (strncmp(argv[argindex], "sdc", arglength) == 0)) {
		if (argc < 3) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " sdc IEEE_address ?IEEE_address? ...\"\n", (char *) NULL);
			goto IEEE_Help;
		}
		argindex++;

		opto_open(IEEE_DEVICE);
		for (; argindex < argc; argindex++) {
			ret = reset(argv[argindex]);
			if (ret < 0) {
				opto_close();
				sprintf(interp->result, "%d", ret);
				Tcl_AppendResult(interp, "# Error sending to IEEE ...", argv[0],
								 argv[1], argv[argindex], (char *) NULL);
				return TCL_ERROR;
			}
		}
		opto_close();
		return TCL_OK;

/*** inquire ***/
	} else if ((argchar == 'i') && (strncmp(argv[argindex], "inquire", arglength) == 0)) {
		if (argc != 4) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " inquire IEEE_address IEEE_string\"\n", (char *) NULL);
			goto IEEE_Help;
		}
		argindex++;

		opto_open(IEEE_DEVICE);
		ret = i3e_output(argv[argindex], argv[argindex + 1], strlen(argv[argindex + 1]));
		if (ret < 0) {
			opto_close();
			sprintf(interp->result, "%d", ret);
			Tcl_AppendResult(interp, "# Error sending to IEEE ...", argv[0],
							 argv[1], argv[argindex], argv[argindex + 1], (char *) NULL);
			return TCL_ERROR;
		}
		ret = i3e_input(argv[argindex], IEEE_Buffer, mode);
		opto_close();

		if (ret >= 0) {
			IEEE_Buffer[ret] = 0;
			Tcl_AppendResult(interp, IEEE_Buffer, (char *) NULL);
			return TCL_OK;
		} else {
			interp->result[0] = 0;
			Tcl_AppendResult(interp, "# Error reading from IEEE ...", argv[0],
							 argv[1], argv[argindex], (char *) NULL);
			return TCL_ERROR;
		}

/*** trigger ***/
	} else if ((argchar == 't') && (strncmp(argv[argindex], "trigger", arglength) == 0)) {
		if (argc < 3) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " trigger IEEE_address ?IEEE_address? ...\"\n", (char *) NULL);
			goto IEEE_Help;
		}
		argindex++;

		opto_open(IEEE_DEVICE);
		for (; argindex < argc; argindex++) {
			trigger(argv[argindex]);
		}
		opto_close();
		return TCL_OK;

/*** remote ***/
	} else if ((argchar == 'r') && (strncmp(argv[argindex], "remote", arglength) == 0)) {
		if (argc < 3) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " remote IEEE_address ?IEEE_address? ...\"\n", (char *) NULL);
			goto IEEE_Help;
		}
		argindex++;

		opto_open(IEEE_DEVICE);
		for (; argindex < argc; argindex++) {
			remote(argv[argindex]);
		}
		opto_close();
		return TCL_OK;

/*** talker ***/
	} else if ((argchar == 't') && (strncmp(argv[argindex], "talker", arglength) == 0)) {
		if (argc != 3) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " talker IEEE_address\"\n", (char *) NULL);
			goto IEEE_Help;
		}
		argindex++;

		opto_open(IEEE_DEVICE);
		talker(argv[argindex]);
		opto_close();

		return TCL_OK;

/*** listener ***/
	} else if ((argchar == 'l') && (strncmp(argv[argindex], "listener", arglength) == 0)) {
		if (argc < 3) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " listener IEEE_address ?IEEE_address? ...\"\n", (char *) NULL);
			goto IEEE_Help;
		}
		argindex++;

		opto_open(IEEE_DEVICE);
		for (; argindex < argc; argindex++) {
			listener(argv[argindex]);
		}
		opto_close();
		return TCL_OK;

/*** status ***/
	} else if ((argchar == 's') && (strncmp(argv[argindex], "status", arglength) == 0)) {
		if (argc < 3) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " status IEEE_address ?IEEE_address? ...\"\n", (char *) NULL);
			goto IEEE_Help;
		}
		argindex++;

		opto_open(IEEE_DEVICE);
		for (; argindex < argc; argindex++) {
			ret = status(argv[argindex], &IEEE_Int);
			if (ret >= 0) {
				sprintf(IEEE_Buffer, "%d", IEEE_Int);
				Tcl_AppendResult(interp, argv[argindex], ": ", IEEE_Buffer, "\n", (char *) NULL);
			} else {
				Tcl_AppendResult(interp, "# Error reading from IEEE ...", argv[argindex], (char *) NULL);
				opto_close();
				return TCL_ERROR;
			}
		}
		opto_close();

		return TCL_OK;

/*** kein i3e-Befehl erkannt ... ***/
	} else {
		Tcl_AppendResult(interp, "bad Command\n", (char *) NULL);
		goto IEEE_Help;
	}

  IEEE_Help:
	Tcl_AppendResult(interp, "USAGE:  i3e [option] command IEEE_address [arg ...]\n\n",
					 "        option:  -EOI (default), -CR or -CRLF\n",
					 "        command: alloc, dealloc\n",
					 "                 read, send, inquire, trigger\n",
					 "                 talker, listener, remote, gtl\n",
					 "                 sdc, reset, status", (char *) NULL);
	return TCL_ERROR;
}
