
/* 
 * tclOPTO.c --
 *
 *  This file contains the top-level command routines for the
 *  the Kryo-spezific built-in commands 
 *
 * Copyright (c) 1987-1993 The Regents of the University of California.
 * Copyright (c) 1994-1995 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <tcl.h>

#include "../gpib/gpib_drv.h"

#define IEEE_MAXLEN 1024   /* max length of files to read in for cmd WRITF */

/*
*  Function name : i3e_input
*
*  Description : liest Daten von einem Geraet in einen Puffer. Die 
*                    Adressierung des Talkerers wird ebenfalls erledigt. 
*  Input :       adf  ist die ascii-Adressierungsfolge der Listener.
*            data  ist die Pufferadresse in dem die zu empfangenen   
*                          Daten gespeichert werden.           
*                    ende  Endekennung, zB CRLF
*  Output :      Anzahl der gelesenen Bytes
*                    -1 zurueckgeliefert wenn ein Sendetimeout auftritt
*                    -2 bei einem Empfangstimeout
*/
int  i3e_input(char *adf, char *data, int len)
{
	char filename[256];
	int gpib,readlen=0;

	strcpy(filename, "/dev/gpib/");
	strcat(filename, adf);
	
	gpib = open(filename, O_RDWR|O_SYNC);
	if (gpib != 0) {
		readlen=ib_read(gpib, data, len);
		ib_close(gpib);
	} else {
		printf("ERROR while writing to gpib-device %s\n", adf);
	}

	data[readlen]=0;
	return readlen;
}
	
/*
*  Function name : i3e_output
*
*  Description : gibt Daten an Geraete aus. Die Adressierung der Listen-
*            er wird ebenfalls erledigt.
*  Input :       adf  ist die ascii-Adressierungsfolge der Listener.
*            data  ist die Pufferadresse in dem die zu sendenden   
*                          Daten gespeichert sind.         
*                    anzahl ist die Zahl der Bytes in data
*  Output :      0 => ok
*                    -1 zurueckgeliefert wenn ein Sendetimeout auftritt
*                    -2 bei einem Empfangstimeout
*/
int  i3e_output(char *adf,void *data,int anzahl)
{
	char filename[256];
	int gpib;
	
	strcpy(filename, "/dev/gpib/");
	strcat(filename, adf);
	
	gpib = open(filename, O_RDWR|O_SYNC);

	if (gpib != 0) {
		ib_write(gpib, data, anzahl);
		ib_close(gpib);
	} else {
		printf("ERROR while writing to gpib-device %s\n", adf);
	}
		
	return strlen(data);
}
/* FIXME das hier nur solange das Ding nicht komplett geaendert ist! */
#define IEEE_DEVICE "/dev/null"
void opto_open(char *dev)
{
	printf("opto_open()\n");
}	
void opto_close()
{
	printf("opto_close()\n");
}	
void talker(char *dev)
{
	printf("talker()\n");
}	
void listener(char *dev)
{
	printf("listener()\n");
}	

int reset(char *dev)
{
	printf("reset()\n");
	return 0;
}	

int gtl(char *dev)
{
	printf("gtl()\n");
	return 0;
}

void trigger(char *dev)
{
	printf("trigger()\n");
}

void remote(char *dev)
{
	printf("remote()\n");
}

void power_on(int dev)
{
	printf("power_on()\n");
}

int addsearch(char *dev)
{
	printf("addsearch()\n");
	return 0;
}

int status(char *dev, int *status)
{
	printf("status()\n");
	return 0;
}	

int numaddress(char *IEEE_Devname)
{
	int adr=0,ret=0;

	adr = open (IEEE_Devname,O_RDWR|O_SYNC);

	if ( adr < 0 ) {
		ret=adr;
	} else {
		ret = ioctl(adr,GPIB_NUL);
		close (adr);
	}
	return (ret);
}

void mkdevname(char *str, char *IEEE_Devname)
{
	if (str[0]!='/')
		strcpy(IEEE_Devname,GPIB_PATH), strcat(IEEE_Devname,str);
	else
		strcpy(IEEE_Devname,str);

	if (numaddress(IEEE_Devname) < 0 )
		strcpy (IEEE_Devname,"NULL");
	return;
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_IEEECmd --
 *
 *  This procedure is invoked to communicate with the IEEE-Bus-System
 *      using the optolib.c - Functions 
 *  See the user documentation for details on what it does.
 *
 * Results:
 *  A standard Tcl result.
 *
 * Side effects:
 *  See the user documentation.
 *
 *----------------------------------------------------------------------
 */

int Tcl_IEEECmd(ClientData dummy, Tcl_Interp * interp, int argc, char **argv)
{
	size_t arglength;
	int argindex;
	char argchar = 0;

	int ret, mode = 0x0000;

	char IEEE_Buffer[IEEE_MAXLEN + 1];
	int IEEE_Int;
	char IEEE_Devname[256];
	int IEEE_Dev;
		

	if (argc < 2)
		goto IEEE_Help;

	/* Optionen parsen und aus dem Argumentfeld entfernen: */

	for (argindex = 1; argindex < argc;) {
		if (argv[argindex][0] == '-') {

			/* Hier die Optionen abfragen: */
			arglength = strlen(argv[argindex]);

			if (strncmp(argv[argindex], "-EOI", arglength) == 0) {
				mode = 0x0000;
			} else if (strncmp(argv[argindex], "-CR", arglength) == 0) {
				mode = 0x000d;
			} else if (strncmp(argv[argindex], "-CRLF", arglength) == 0) {
				mode = 0x0d0a;
			} else {
				Tcl_AppendResult(interp, "ERROR: unknown option '", argv[argindex], "'\n", (char *) NULL);
				return TCL_ERROR;
			}

			/* Zeiger auf Argumente verschieben: */
			if (argindex < argc) {
				int i;
				for (i = argindex; i < argc; i++)
					argv[i] = argv[i + 1];
			}
			argc--;
		} else
			argindex++;
	}

	/* Argumentindex wieder auf den Beginn der Argumente setzen */
	argindex = 1;

	/* Nun die Argumente parsen: */

	if (argc < 2) {
		Tcl_AppendResult(interp, "wrong # args:\n", (char *) NULL);
		goto IEEE_Help;
	}
	argchar = argv[argindex][0];
	arglength = strlen(argv[argindex]);

/*** alloc ***/
	if ((argchar == 'a') && (strncmp(argv[argindex], "alloc", arglength) == 0)) {
		if (argc != 2) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " alloc\"", (char *) NULL);
			return TCL_ERROR;
		}
		opto_open(IEEE_DEVICE);
		return TCL_OK;

/*** dealloc ***/
	} else if ((argchar == 'd') && (strncmp(argv[argindex], "dealloc", arglength) == 0)) {
		if (argc != 2) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " dealloc\"", (char *) NULL);
			return TCL_ERROR;
		}
		opto_close();
		return TCL_OK;

/*** gtl ***/
	} else if ((argchar == 'g') && (strncmp(argv[argindex], "gtl", arglength) == 0)) {
		if (argc < 3) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " gtl IEEE_address ?IEEE_address? ...\"", (char *) NULL);
			return TCL_ERROR;
		}
		argindex++;

		opto_open(IEEE_DEVICE);
		
		for (; argindex < argc; argindex++) {
			ret = gtl(argv[argindex]);
			if (ret < 0) {
				opto_close();
				sprintf(interp->result, "%d", ret);
				Tcl_AppendResult(interp, "# Error sending to IEEE ...", argv[0],
								 argv[1], argv[argindex], (char *) NULL);
				return TCL_ERROR;
			}
		}
		opto_close();
		return TCL_OK;

/*** read ***/
	} else if ((argchar == 'r') && (strncmp(argv[argindex], "read", arglength) == 0)) {
		if (argc != 3) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " read IEEE_address\"\n", (char *) NULL);
			goto IEEE_Help;
		}
		
		argindex++;

		mkdevname(argv[argindex], IEEE_Devname);
	
		if (!strcmp("NULL",IEEE_Devname)) {
			Tcl_AppendResult(interp, "ERROR: cannot open device", (char *) NULL);
			return TCL_ERROR;
		}
		
		if ((IEEE_Dev=open(IEEE_Devname,O_RDWR|O_SYNC))<0) {
			Tcl_AppendResult(interp, "ERROR: cannot open device", (char *) NULL);
			return TCL_ERROR;
		}
		
		ret = read(IEEE_Dev, IEEE_Buffer, IEEE_MAXLEN);
		
		close(IEEE_Dev);

		if (ret >= 0) {
			IEEE_Buffer[ret] = 0;	/* gesendete Zeichenkette terminieren */
			Tcl_AppendResult(interp, IEEE_Buffer, (char *) NULL);
			return TCL_OK;
		} else {
			interp->result[0] = 0;
			Tcl_AppendResult(interp, "# Error reading from IEEE ...", argv[0],
							 argv[argindex - 1], argv[argindex], (char *) NULL);
			return TCL_ERROR;
		}

/*** reset ***/
	} else if ((argchar == 'r') && (strncmp(argv[argindex], "reset", arglength) == 0)) {
		if (argc < 3) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " reset IEEE_address ?IEEE_address? ...\"\n", (char *) NULL);
			goto IEEE_Help;
		}
		argindex++;

		return TCL_OK;

/*** sdc ***/
	} else if ((argchar == 's') && (strncmp(argv[argindex], "sdc", arglength) == 0)) {
		if (argc < 3) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " sdc IEEE_address ?IEEE_address? ...\"\n", (char *) NULL);
			goto IEEE_Help;
		}
		argindex++;

		opto_open(IEEE_DEVICE);
		
		for (; argindex < argc; argindex++) {
			ret = reset(argv[argindex]);
			
			if (ret < 0) {
				opto_close();
				
				sprintf(interp->result, "%d", ret);
				Tcl_AppendResult(interp, "# Error sending to IEEE ...", argv[0],
								 argv[1], argv[argindex], (char *) NULL);
				return TCL_ERROR;
			}
		}
		opto_close();
		return TCL_OK;

/*** query ***/
	} else if ((argchar == 'q') && (strncmp(argv[argindex], "query", arglength) == 0)) {
		if (argc != 4) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " inquire IEEE_address IEEE_string\"\n", (char *) NULL);
			goto IEEE_Help;
		}
		
		argindex++;

		mkdevname(argv[argindex], IEEE_Devname);
	
		if (!strcmp("NULL",IEEE_Devname)) {
			Tcl_AppendResult(interp, "ERROR: cannot open device", (char *) NULL);
			return TCL_ERROR;
		}
		
		if ((IEEE_Dev=open(IEEE_Devname,O_RDWR|O_SYNC))<0) {
			Tcl_AppendResult(interp, "ERROR: cannot open device", (char *) NULL);
			return TCL_ERROR;
		}
		
		argindex++;
		
		ret = write(IEEE_Dev, argv[argindex], strlen(argv[argindex]));
		
		if (ret < 0) {
			close(IEEE_Dev);
			
			sprintf(interp->result, "%d", ret);
			Tcl_AppendResult(interp, "# Error sending to IEEE ...", argv[0],
							 argv[1], argv[argindex], argv[argindex + 1], (char *) NULL);
			
			return TCL_ERROR;
		}

		usleep(100);
		
		ret = read(IEEE_Dev, IEEE_Buffer, IEEE_MAXLEN);
		
		close(IEEE_Dev);
		
		if (ret <0) {
			interp->result[0] = 0;
			Tcl_AppendResult(interp, "# Error reading from IEEE ...", argv[0],
							 argv[argindex - 1], argv[argindex], (char *) NULL);
			return TCL_ERROR;
		}

		IEEE_Buffer[ret] = 0;	/* gesendete Zeichenkette terminieren */
		Tcl_AppendResult(interp, IEEE_Buffer, (char *) NULL);
		return TCL_OK;

/*** trigger ***/
	} else if ((argchar == 't') && (strncmp(argv[argindex], "trigger", arglength) == 0)) {
		if (argc < 3) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " trigger IEEE_address ?IEEE_address? ...\"\n", (char *) NULL);
			goto IEEE_Help;
		}
		argindex++;

		opto_open(IEEE_DEVICE);
		
		for (; argindex < argc; argindex++) {
			trigger(argv[argindex]);
		}
		opto_close();
		
		return TCL_OK;

/*** remote ***/
	} else if ((argchar == 'r') && (strncmp(argv[argindex], "remote", arglength) == 0)) {
		if (argc < 3) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " remote IEEE_address ?IEEE_address? ...\"\n", (char *) NULL);
			goto IEEE_Help;
		}
		argindex++;

		opto_open(IEEE_DEVICE);
		for (; argindex < argc; argindex++) {
			remote(argv[argindex]);
		}
		opto_close();
		return TCL_OK;

/*** talker ***/
	} else if ((argchar == 't') && (strncmp(argv[argindex], "talker", arglength) == 0)) {
		if (argc != 3) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " talker IEEE_address\"\n", (char *) NULL);
			goto IEEE_Help;
		}
		argindex++;

		opto_open(IEEE_DEVICE);
		talker(argv[argindex]);
		opto_close();

		return TCL_OK;

/*** listener ***/
	} else if ((argchar == 'l') && (strncmp(argv[argindex], "listener", arglength) == 0)) {
		if (argc < 3) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " listener IEEE_address ?IEEE_address? ...\"\n", (char *) NULL);
			goto IEEE_Help;
		}
		argindex++;

		opto_open(IEEE_DEVICE);
		for (; argindex < argc; argindex++) {
			listener(argv[argindex]);
		}
		opto_close();
		return TCL_OK;

/*** status ***/
	} else if ((argchar == 's') && (strncmp(argv[argindex], "status", arglength) == 0)) {
		if (argc < 3) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " status IEEE_address ?IEEE_address? ...\"\n", (char *) NULL);
			goto IEEE_Help;
		}
		argindex++;

		opto_open(IEEE_DEVICE);
		for (; argindex < argc; argindex++) {
			ret = status(argv[argindex], &IEEE_Int);
			if (ret >= 0) {
				sprintf(IEEE_Buffer, "%d", IEEE_Int);
				Tcl_AppendResult(interp, argv[argindex], ": ", IEEE_Buffer, "\n", (char *) NULL);
			} else {
				Tcl_AppendResult(interp, "# Error reading from IEEE ...", argv[argindex], (char *) NULL);
				opto_close();
				return TCL_ERROR;
			}
		}
		opto_close();

		return TCL_OK;

/*** write ***/
	} else if ((argchar == 'w') && (strncmp(argv[argindex], "write", arglength) == 0)) {
		if (argc != 4) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " write IEEE_address IEEE_string\"\n", (char *) NULL);
			goto IEEE_Help;
		}
		argindex++;
		
		mkdevname(argv[argindex], IEEE_Devname);
	
		if (!strcmp("NULL",IEEE_Devname)) {
			Tcl_AppendResult(interp, "ERROR: cannot open device", (char *) NULL);
			return TCL_ERROR;
		}
		
		if ((IEEE_Dev=open(IEEE_Devname,O_RDWR|O_SYNC))<0) {
			Tcl_AppendResult(interp, "ERROR: cannot open device", (char *) NULL);
			return TCL_ERROR;
		}
		
		argindex++;
		
		ret = write(IEEE_Dev, argv[argindex], strlen(argv[argindex]));
		
		close(IEEE_Dev);

		if (ret < 0) {
			sprintf(interp->result, "%d", ret);
			Tcl_AppendResult(interp, "# Error sending to IEEE ...", argv[0],
							 argv[1], argv[argindex], argv[argindex + 1], (char *) NULL);
			return TCL_ERROR;
		}
		return TCL_OK;

/*** kein i3e-Befehl erkannt ... ***/
	} else {
		Tcl_AppendResult(interp, "bad Command\n", (char *) NULL);
		goto IEEE_Help;
	}

  IEEE_Help:
	Tcl_AppendResult(interp, "USAGE:  i3e [option] command IEEE_address [arg ...]\n\n",
					 "        option:  -EOI (default), -CR or -CRLF\n",
					 "        command: alloc, dealloc\n",
					 "                 read, send, inquire, trigger\n",
					 "                 talker, listener, remote, gtl\n",
					 "                 sdc, reset, status", (char *) NULL);
	return TCL_ERROR;
}
