/* 
 * tclopto_LeCroy.c --
 *
 *  This file contains the top-level command routines for the
 *  the Kryo-spezific built-in commands 
 *
 * Copyright (c) 1987-1993 The Regents of the University of California.
 * Copyright (c) 1994-1995 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#define IEEE_DEVICE "/dev/opto0"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <fcntl.h>
#include <unistd.h>

#include <tcl.h>

#include "optolib.h"
#include "rwadat.h"

/*
 * Prototypen
 */

char * Exp_Config_Dir();
	
/***************************
* globale Zeiger           *
***************************/

/* Changed by JL */
char sinfo[32000];

struct WAVEDESC {
	char descriptor_name[16];
	char template_name[16];
	short int comm_type;
	short int comm_order;
	int wave_descriptor;
	int user_text;
	int res_desc1;
	int trigtime_array;
	int ris_time_array;
	int res_array1;
	int wave_array_1;
	int wave_array_2;
	int res_array2;
	int res_array3;
	char instrument_name[16];
	int instrument_number;
	char trace_label[16];
	short int reserved1;
	short int reserved2;
	int wave_array_count;
	int pnts_per_screen;
	int first_valid_pnt;
	int last_valid_pnt;
	int first_point;
	int sparsing_factor;
	int segment_index;
	int subarray_count;
	int sweeps_per_acq;
	int obsolete1;
	float vertical_gain;
	float vertical_offset;
	float max_value;
	float min_value;
	short int nominal_bits;
	short int nom_subarray_count;
	float horiz_interval;
	double horiz_offset;
	double pixel_offset;
	char vertunit[48];
	char horunit[48];
	short int reserved3;
	short int reserved4;
/*char trigger_time[16]; */
	double seconds;				/* trigger_time.start */
	unsigned char minutes;
	unsigned char hours;
	unsigned char days;
	unsigned char months;
	int year;					/* trigger_time.end */
	float acqduration;
	short int record_type;
	short int processing_done;
	short int reserved5;
	short int ris_sweeps;
	short int timebase;
	short int vert_coupling;
	float probe_att;
	short int fixed_vert_gain;
	short int bandwidth_limit;
	float vertical_vernier;
	float acq_vert_offset;
	short int wave_source;
	char dummy[100];
};

/********************************************************************************
*                                                                               *
*  rdlc liest Daten vom neuen LECROY ueber I3E in eine Datei               		*
*                                                                               *
*  so gehts: rdlc(I3e_add,spnr,dma,all,trwait,steprate,blocksize,pfad,retmsg)   *
*  i3e_add = IEEE Address                                                       *
*  snr     = C1/C2/EA/EB/MC/MD/FE/FF ist die Bezeichnung eines Lecroyspeichers	*
*  trwait  = y/n/f mit  y/n wird bestimmt ob auf einen Trigger                  *
*            gewartet wird vor dem auslesen oder nicht                          *
*            mit f wid eine Fehlermeldung ausgegeben wenn kein                  *
*            Trigger vorhanden\n");                                             *
*  steprate      1-?  nur jeder steprate-te Wert wird vom Lecroy gelese         *
*  blocksize es werden Bloecke der Laenge blocksize eingelesen				*
			 fuer blocksize=0 wird alles in einem Block eingelesen			*
*  pfad    = vollst"andiger Pfad, wo die Daten gespeichert                      *
*            werden sollen: z.B. e:\daten\Lecroy.dat                            *
*  files   = ist eine Liste von ASCII-Files,deren Inhalt in der Sonderinfo      *
*	     	abgespeichert wird.Die Files werden durch @ voneinander getrennt.	*
*	     	z.B. rsgo.s0@pulser.p0                                             	*
*  retmsg  = in diesen Puffer werden Meldungen zurueckgegeben                   *
*                                                                               *
*********************************************************************************/

int rdlc(char *add, char *snr, char trwait, int step, int blocksize, char *pfad, char *files, char *retmsg)
{
	struct WAVEDESC wavedesc;
	int fehler = 0, handl2, i;
	int anz = 0l, trc;
	char coupling[10], command[200], filename[50], xeinh[4], yeinh[4];
	char *sptr, ttime[100], buf[1300];
	int tbm = 0, tbe, vgm = 0, vge, failcount=0;
	double trd, sec;

	unsigned char *inputbuf = NULL;
	char *wave_buffer = NULL;

	opto_open(IEEE_DEVICE);

	if (trwait == 'y') {		/* solange warten bis Lecroy getriggert */
		do {
			/* String senden */
			sprintf(command, "INR?");
			fehler = i3e_output(add, command, strlen(command));

			/* Empfangen des Status-Bytes */
			i3e_input(add, buf, EOI);

			/* Falls das Bit #0 des Statusbytes = 1 ist wurde ein neues Signal
			 * aufgenommen */			
			if ((1 & atoi(&(buf[4]))) == 0) {
				/* Zurueckschalten in den Handbetrieb */
				fehler = gtl(add);
			} else break;
		
			/* Falls eine halbe Minute kein Trigger kommt weitermachen */
			if (failcount++ > 30) break;
		   	
			/* jetzt anderen die Mglichkeit geben, auch auf den I3E-Bus zuzugreifen */
			opto_close();
			sleep(1);
			opto_open(IEEE_DEVICE);

		} while (1);

	} else if (trwait == 'f') {	/* aussteigen wenn kein Trigger */
		/* String senden */
		sprintf(command, "INR?");
		fehler = i3e_output(add, command, strlen(command));

		/* Empfangen des Status-Bytes */
		i3e_input(add, buf, EOI);

		if ((1 & atoi(&(buf[4]))) == 0) {
			sprintf(retmsg, "kein Trigger\n");
			return -2;
		};
	};
	
	/* Schrittweiten von 0 abfangen */
	if (step==0) step=1;

	/* Datenformat festlegen */
	/* Erst HighByte dann LowByte */
	/* Speicherbereich selektieren festlegen */
	/* WAVEDESC anfordern */
	sprintf(command, "CFMT IND0,BYTE,BIN;CORD LO;\
			WFSU SP,%i,FP,0,NP,0;%s:WF? DESC", step, snr);
	fehler = i3e_output(add, command, strlen(command));

	/* WAVEDESC lesen */
	anz = i3e_input(add, buf, EOI);
	memcpy(&wavedesc, &buf[13], sizeof(wavedesc));

	strncpy(xeinh, wavedesc.horunit, 3);
	xeinh[3] = 0;
	strncpy(yeinh, wavedesc.vertunit, 3);
	yeinh[3] = 0;

	/* Falls ein Wert von 0 fuer blocksize uebergeben wurde, wird der
	 * Datensatz in einem Stueck eingelesen */

	if (blocksize == 0 || blocksize > wavedesc.pnts_per_screen)
		blocksize = wavedesc.pnts_per_screen;	/* nur einen Block lesen */

	/* Speicher fuer Eingabepuffer reservieren */
	if (!(inputbuf = malloc(blocksize + 14))) {
		fprintf(stderr, "Out of Memory \n Can't allocate %d Bytes for InputBuffer\n",
				blocksize + 14);
		return -1;
	}
	
	/* Speicher fuer Oszilloskopdaten reservieren */
	if (!(wave_buffer = malloc(wavedesc.pnts_per_screen))) {
		fprintf(stderr, "Out of Memory \n Can't allocate %d Bytes for Waveform\n",
				wavedesc.pnts_per_screen);
		return -1;
	}
	
	
	
	for (i = 0; i < wavedesc.pnts_per_screen; i += blocksize * step) {
		opto_close();
		opto_open(IEEE_DEVICE);

		/* Datenbereich und -laenge setzen und Daten anfordern*/
		sprintf(command, "WFSU SP,%i,FP,%i,NP,%i;%s:WF? DAT1",
				step, i, blocksize,snr);	/* bei WFSU ,SN,0 entfernt! */
		fehler = i3e_output(add, command, strlen(command));

		/* Daten lesen */
		anz = i3e_input(add, inputbuf, EOI);
		
		if (blocksize < wavedesc.pnts_per_screen) {
			printf(".");
			fflush(NULL);
		}

		memcpy(&wave_buffer[i / step], &inputbuf[14], anz - 14);
	}

	if (blocksize < wavedesc.pnts_per_screen)
		printf("\n");

	opto_close();

	/* DESC AUSWERTEN */

	sinfo[0] = 0;

	switch (wavedesc.timebase % 3) {	/* Manntisse Timebase*/
	case 0:
		tbm = 1; break;
	case 1:
		tbm = 2; break;
	case 2:
		tbm = 5; break;
	}
	tbe = wavedesc.timebase / 3 - 12;	/* Exponent Timebase*/

	switch (wavedesc.fixed_vert_gain % 3) {		/* Manntisse VDiv*/
	case 0:
		vgm = 1; break;
	case 1:
		vgm = 2; break;
	case 2:
		vgm = 5; break;
	}
	vge = wavedesc.fixed_vert_gain / 3 - 6;		/* Exponent VDiv*/

/* Triggerdelay */
	trd = wavedesc.horiz_offset;

/* Triggerchannel */
	trc = -(int) (trd / (double) wavedesc.horiz_interval);

/* Channelcoupling */
	switch (wavedesc.vert_coupling) {
	case 0:
		strcpy(coupling, "DC_50_Ohms"); break;
	case 1:
		strcpy(coupling, "ground"); break;
	case 2:
		strcpy(coupling, "DC_1MOhm"); break;
	case 3:
		strcpy(coupling, "ground"); break;
	case 4:
		strcpy(coupling, "AC_1MOhm"); break;
	};

/* Triggerzeit ermitteln */
	sec = wavedesc.seconds;		/* Sekunden ermitteln */
	sprintf(ttime, "TRT= %02hd:%02hd:%02hd %hd %hd %hd TRS= %e",
			(short int) wavedesc.hours,
			(short int) wavedesc.minutes,
			(short int) sec,
			(short int) wavedesc.days,
			(short int) wavedesc.months,
			wavedesc.year,
			sec);

	sprintf(sinfo, "\nTB=%dE%duS VD=%gmV TRD=%eS LBSC=0\nTRC=%d CHCOUP=%s\n%s\n\n",
			tbm, tbe + 6, (double) (vgm * pow(10, vge + 3)), trd, trc,
			coupling, ttime);

	/* ASCII-Files anhaengen */
	sptr = strtok(files, "@; ");
	while (sptr != NULL) {
		strcpy(filename, sptr);
		
		/* zuerst versuchen, das file im aktuellen Verzeichnis zu lesen */
		handl2 = open(filename, O_RDONLY);
		if (handl2 > 0) {
			anz = read(handl2, buf, 1024l);
			buf[anz] = 0;
			strcat(sinfo, filename);
			strcat(sinfo, ":\n");
			strcat(sinfo, buf);
			close(handl2);
		} else {
			/* dann versuchen, das File aus dem ConfigDir zu lesen */
			strcpy(filename, Exp_Config_Dir());
			strcat(filename, "/params/");
			strcat(filename, sptr);
			handl2 = open(filename, O_RDONLY);
			if (handl2 > 0) {
				anz = read(handl2, buf, 1024l);
				buf[anz] = 0;
				strcat(sinfo, sptr);
				strcat(sinfo, "+:\n  ");
				strcat(sinfo, buf);
				close(handl2);
			} 
		}
		sptr = strtok(NULL, "@; ");
	};
	
	if (failcount>30) {
		for (i = 0; i < wavedesc.pnts_per_screen / step; i++)
			wave_buffer[i]=0;
	}

	fehler = wadat(NULL, wave_buffer, 'b', 'b', pfad, wavedesc.first_valid_pnt,
				   wavedesc.pnts_per_screen / step,
				   1.0l, wavedesc.vertical_gain,
				   0.0l, -wavedesc.vertical_offset,
				   xeinh, yeinh,
				   0.0l, (wavedesc.pnts_per_screen - 1.0l) *
				   wavedesc.horiz_interval,
				   'L', (void *) sinfo, (int) strlen(sinfo), 0);

	sprintf(retmsg, "%d Datenbytes uebertragen\n", wavedesc.pnts_per_screen / step);

	free(inputbuf);
	free(wave_buffer);

	return fehler;
};

/*
 *----------------------------------------------------------------------
 *
 * Tcl_ReadLeCroyCmd --
 *
 *  This procedure is invoked to communicate with the IEEE-Bus-System
 *      using the optolib.c - Functions 
 *  See the user documentation for details on what it does.
 *
 * Results:
 *  A standard Tcl result.
 *
 * Side effects:
 *  See the user documentation.
 *
 *----------------------------------------------------------------------
 */

int Tcl_ReadLeCroyCmd(ClientData dummy, Tcl_Interp * interp, int argc, char **argv)
{
	int steprate = 1, blocksize = 0, fehler = 0;
	char trwait = 'n', retmsg[100];
	int argindex, i;

	bool READ_OLD_LECROY = FALSE;	/* Option: -old */

	/* Optionen parsen und aus dem Argumentfeld entfernen: */

	for (argindex = 1; argindex < argc;) {
		if (argv[argindex][0] == '-') {

			/* Hier die Optionen abfragen: */
			if (strcmp(argv[argindex], "-old") == 0) {
				READ_OLD_LECROY = TRUE;
			} else {
				Tcl_AppendResult(interp, "ERROR: unknown option '", argv[argindex], "'\n", (char *) NULL);
				return TCL_ERROR;
			}

			/* Zeiger auf Argumente verschieben: */
			if (argindex < argc) {
				for (i = argindex; i < argc; i++)
					argv[i] = argv[i + 1];
			}
			argc--;
		} else
			argindex++;
	}

	if (argc < 6)
		goto ReadLeCroy_Help;

	/* Argumentindex wieder auf den Beginn der Argumente setzen */
	argindex = 1;

	if (sscanf(argv[argindex + 3], "%i", &steprate) == 0) {
		Tcl_AppendResult(interp, "ERROR: value <steprate> is not of type integer\n", (char *) NULL);
		return TCL_ERROR;
	}
	if (sscanf(argv[argindex + 4], "%i", &blocksize) == 0) {
		Tcl_AppendResult(interp, "ERROR: value <blocksize> is not of type integer\n", (char *) NULL);
		return TCL_ERROR;
	}
	trwait = argv[argindex + 2][0];

	fehler = rdlc(argv[argindex], argv[argindex + 1],
				  trwait, steprate, blocksize,
				  argv[argindex + 5], argv[argindex + 6],
				  retmsg);

	Tcl_AppendResult(interp, retmsg, (char *) NULL);

	return TCL_OK;

  ReadLeCroy_Help:
	Tcl_AppendResult(interp, "USAGE: rdlc <LeCroy_I3ELabel> <source> <trigmode> <steprate> <blocksize> <outfile> [parameterfiles] \n ",
					 (char *) NULL);
	return TCL_ERROR;
}
