/* 
 * tclopto_DAC.c --
 *
 *  This file contains the top-level command routines for the
 *  the Kryo-spezific built-in commands 
 *
 * Copyright (c) 1987-1993 The Regents of the University of California.
 * Copyright (c) 1994-1995 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#define MUX_DEVICE "/dev/opto0"
#define MESSMUX_DEVICE  "/dev/opto1"

extern long DAC_Word(int mode, int filter, double dwert);

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>
#include <time.h>
#include <ctype.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/times.h>

#include <tcl.h>

#include "optolib.h"
#include "tclmod_MUX.h"

#define PKT2       0
#define PKT4       1

char *Exp_Config_Dir();

/************************************************************************
*																	   	*
* muxaddset:setzt die Input- und Outputadresse des MUX. 				*
* Eingabe:	inadd ist die Inputadresse									*
*			outadd ist die Outputadresse								*
* Ergebnis: 0 wenn alles ok, -1 (=Sendetimeout), -2 wenn die MUXoptoadd	*
*			nicht ermittelt werden konnte.								*
*																		*
************************************************************************/
int muxaddset(int inadd, int outadd)
{
	int optoadd;
	int fehlnr = 0;
	unsigned send = 0x8000, oldset, newset;

	optoadd = addsearch("MUX");	/* Optoadd vom MUX suchen */
	if (optoadd < 0)
		return -2;

	fehlnr = cbitset(optoadd, 1);	/* MUX-Addset-Modus */
	fehlnr = cbitclr(optoadd, 2);
	oldset = getbm(1);
	if (inadd < 0) {
		inadd = (oldset >> 8) & 15;
	}
	if (outadd < 0) {
		outadd = oldset & 15;
	}
	
	newset = ((inadd & 0xf) << 8) + (outadd & 0xf);
	setbm(1, newset);
	send |= newset;
	fehlnr = wordout(optoadd, send);
	/*	fehlnr = wordout(optoadd, send);*/
	fehlnr = cbitclr(optoadd, 1);	/* MUX-Data-Modus */

	return fehlnr;
};

/************************************************************************
*																	   	*
* muxbit :veraendert Bits an einer Outputbuchse  						*
* Eingabe:	muxadd 	ist die Adresse der Muxbuchse						*
*			bits 	gibt an welche Bits veraendert werden sollen:		*
*					z.B. bits = 0x3 => 0 & 1 werden verandert 2-15 		*
*					bleiben unverndert.								*
*			modus   1 => Bits werden gesetzt							*
*					0 => Bits werden geloescht							*
*					2 => alle bits werden bernommen					*
*					3 => Ausgabe eines Pulses an den angegebenen Bits 	*
*					4 => Bits werden gesetzt und ...-Bits werden 		*
*						 geloescht d.h. es muss eine weitere unsigned   *
*						 Zahl der zu loeschenden Bits angegeben werden	*
* Ergebnis: 0 wenn alles ok, -1 (=Sendetimeout), -2 wenn die MUXoptoadd	*
*			nicht ermittelt werden konnte.								*
*																		*
************************************************************************/
int muxbit(int muxadd, unsigned bit, int modus,...)
{
	int optoadd;
	int fehlnr = 0;
	unsigned mbit, nbit, bitc;
	va_list argpoint;

	optoadd = addsearch("MUX");	/* Optoadd vom MUX suchen */
	if (optoadd < 0)
		return -2;

	muxaddset(-1, muxadd);		/* Muxoutput neu setzen */

	mbit = getbm(16 + muxadd);	/* Bits setzen */
	switch (modus) {
	case 0:
		nbit = mbit & (~bit);
		break;
	case 1:
		nbit = mbit | bit;
		break;
	case 3:
		nbit = (mbit & (~bit)) | ((~mbit) & bit);
		break;
	case 4:
		va_start(argpoint, modus);
		bitc = va_arg(argpoint, unsigned);
		va_end(argpoint);
		nbit = mbit | bit;
		nbit = nbit & (~bitc);
		break;
	default:
		nbit = bit;
	};
	if (modus == 3) {			/* kurzen Puls anlegen */
		fehlnr = wordout(optoadd, nbit);
		nbit = mbit;
	} else
		setbm(16 + muxadd, nbit);

	fehlnr = wordout(optoadd, nbit);

	return fehlnr;
};

/************************************************************************************
*																					*
* muxin : liest Mux-Buchse															*
*       Eingabe:	muxadd 	ist die Adresse der Muxbuchse							*
*																					*
* Ergebnis: 0 wenn alles ok, -1 (=Sendetimeout), -2 wenn die MUXoptoadd				*
*			nicht ermittelt werden konnte.											*
*																					*
*************************************************************************************/

int muxin(int muxadd, int *wert)
{
	int optoadd;
	int fehlnr = 0;

	optoadd = addsearch("MUX");	/* Optoadd vom MUX suchen */
	if (optoadd < 0)
		return -2;

	muxaddset(muxadd, -1);		/* Muxinput neu setzen */

	fehlnr = wordin(optoadd, wert);

	return fehlnr;
};

/************************************************************************************
*																					*																			 																					 
*  	muxstat:  Erzeugt wenn noch nicht vorhanden eine Status und eine Protokoll- 	*
*			  datei fuer eine MUXoutputbuchse. Die Bitkombination wird ausgegeben	*
*																					*																			 																					 
*	so gehts: muxstat(bnr,string)													*
*	bnr  	 = Buchsennummer (0-13)													*
*	string   = ein Text der wenn vorhanden anstelle der Bitkombination 				*
*			   ausgegeben wird.														*
*																					*
*************************************************************************************/
void muxstat(int bnr, char *string)
{
	char prpfad[100] =
	{""};					/* Protokollfile, die Buchsennummer wird noch angehaengt */
	char stpfad[100] =
	{""};					/* Statusfile, die Buchsennummer wird noch angehaengt */
	char dumstr[100], *tstr, folge[100] =
	{"&"};
	long t;
	FILE *stream;
	unsigned mbit;

	strcat(stpfad, Exp_Config_Dir());
	strcat(stpfad, "/params/mux.s");
	
	strcat(prpfad, Exp_Config_Dir());
	strcat(prpfad, "/params/mux.p");
	
	sprintf(dumstr, "%d", bnr);
	strcat(stpfad, dumstr);		/* Nummer an Statusfile anhaengen */
	strcat(prpfad, dumstr);		/* Nummer an Protokollfile anhaengen */

	/* Zeit und Datum erzeugen */
	t = time(NULL);
	tstr = ctime(&t);
	strncpy(dumstr, tstr + 8l, 3);	/* Tag kopieren */
	strncpy(dumstr + 3l, tstr + 4l, 4);		/* Monat kopieren */
	strncpy(dumstr + 7l, tstr + 22l, 2);	/* Jahr */
	strncpy(dumstr + 9l, tstr + 10l, 10);	/* uhrzeit */
	dumstr[19] = 0;

	if (string == NULL) {
		mbit = getbm(16 + bnr);	/* Bitmuster holen */
		sprintf(folge, "0x%0x", mbit);

	} else {
		strcpy(folge, string);
	};

	stream = fopen(stpfad, "w+");	/* Statusfile schreiben oeffnen */
	if (stream != NULL) {
		fprintf(stream, "%s %s\n", folge, dumstr);
		fclose(stream);
	};
/*  
 * stream = fopen( prpfad,"a+"); 
 * if (stream != NULL){
 * fprintf(stream,"%s %s\n",folge,dumstr);
 * fclose(stream);
 * };
 */
};

/* Aus unerkl"arlichen Gr"unden muss der Befehl min. 100 x gesendet werden *
 * *  Kalte L"otstelle ???                                                    */

#define  MESSMUX_REPEAT 125

/***********************************************************************/
/* Bedient den Messstellenmultiplexer. In modnr wird die Nummer des    */
/* Multiplexermoduls angegeben (zur Zeit nur 0 u 1 erlaubt),in stelle  */
/* die Messstelle und in art die Messart 0==2Punktmessung,1==4Punktmes. */
/***********************************************************************/
int messmultiplex(int modnr, int stelle, int art, int kanalnr)
{
	int i;
	int portout=0, fehlnr=0;

	if (modnr == 0) {
		switch (art) {
		case PKT2:
			portout = ((stelle + 1) / 2) |	/* setzt die untersten 4-Bit */
				8 * (stelle % 2);
			break;
		case PKT4:
			portout = stelle | 0x8;
		}
		for (i = 0; i < MESSMUX_REPEAT; i++)
			fehlnr = optosend(kanalnr, portout);	/* Messtelle umschalten */
	}
	if (modnr == 1) {
		switch (art) {
		case PKT2:
			portout = stelle;
			break;
		case PKT4:
			portout = stelle + 10;
		}
		for (i = 0; i < MESSMUX_REPEAT; i++)
			fehlnr = optosend(kanalnr, 1 * 256 + portout);	/* Messtelle umschalten */
	}
	return fehlnr;
}

/*
 * 
 * int main(int argc,char* argv[])
 * {
 * 
 * int modnr,stelle,art,kanalnr,fehlnr;
 * if(argc < 5){
 * puts("so gehts: messmux  modulnummer messstelle art kanalnummer");
 * return -1;
 * }
 * modnr=atoi(argv[1]);
 * stelle=atoi(argv[2]);
 * art=atoi(argv[3]);
 * kanalnr=atoi(argv[4]);
 * 
 * opto_open("/dev/optosys2");
 * printf("modnr=%d stelle=%d art=%d kanalnr=%d\n",modnr,stelle,art,kanalnr);
 * fehlnr=messmultiplex(modnr,stelle,art,kanalnr);
 * printf("fehlnr=%d\n",fehlnr);
 * opto_close();
 * }
 * 
 */

/*
 *----------------------------------------------------------------------
 *
 * Tcl_MUXCmd --
 *
 *  This procedure is invoked to communicate with the 16x16-Bit MUX
 *      using the 
 *  See the user documentation for details on what it does.
 *
 * Results:
 *  A standard Tcl result.
 *
 * Side effects:
 *  See the user documentation.
 *
 *----------------------------------------------------------------------
 */

int Tcl_MUXCmd(ClientData dummy, Tcl_Interp *interp, int argc, char **argv)
{
	int argindex;
	char argchar;
	
	int buchse, modus, wert = 0, ret;
	unsigned bits = 0, bit, bitc = 0;

	if (argc < 2)
		goto MUX_Help;

	argindex = 1;
	argchar = argv[argindex][0];

	/*** set ***/
	if ((argchar == 's') && (strcmp(argv[argindex], "set") == 0)) {
		if (argc < 5) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
						" set MUX_CHANNEL mode bit ?bit? (or) ?mode bit? ...\"\n", (char *) NULL);
			goto MUX_Help;
		}
		argindex++;

		opto_open(MUX_DEVICE);
		buchse = addsearch(argv[argindex]);	/* Buchsennummer holen */
		
		argindex++;
		
		switch (argv[argindex][0]) {	/* Modus ermitteln */
		case 'C':
		case 'c':
			modus = CLR;
			break;
		case 'S':
		case 's':
			modus = SET;
			break;
		case 'P':
		case 'p':
			modus = PULS;
			break;
		case 'E':
		case 'e':
			modus = WRITE;
			break;
		default:
			Tcl_AppendResult(interp, "unknown MUX-Command: should be \"", argv[0],
							 " c, s, p or e\"\n", (char *) NULL);
			goto MUX_Help;
		}

		for (; argindex < argc; argindex++) {
			if (argv[argindex][0] == 'c' || argv[argindex][0] == 'C') {
				modus = SETCLR;
			};
			if (modus != SETCLR) {
				bit = strtol(argv[argindex], NULL, 0);		/* Bits holen */
				if (isdigit(argv[argindex][0]) && (bit < 16) && (modus != WRITE)) {
					bit = 1 << bit;
				};
				
				bits |= bit;
				
			} else {
				bit = strtol(argv[argindex], NULL, 0);		/* Bits holen */
				if (isdigit(argv[argindex][0]) && (bit < 16) && (modus != WRITE)) {
					bit = 1 << bit;
				};
				
				bitc |= bit;
			};
		};

		if (modus != SETCLR) {
			ret = muxbit(buchse, bits, modus);
		} else {
			ret = muxbit(buchse, bits, modus, bitc);
		}
		if (ret != 0)
			muxstat(buchse, "Fehler beim MUX setzen");
		else
			muxstat(buchse, NULL);
		opto_close();

		if (ret < 0) {
			Tcl_AppendResult(interp, "Timeout while setting MUX", (char *) NULL);
			return TCL_ERROR;
		}
		return TCL_OK;
		
	/*** read ***/
	} else if ((argchar == 'r') && (strcmp(argv[argindex], "read") == 0)) {
		if (argc != 3) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " read MUX_CHANNEL\"", (char *) NULL);
			goto MUX_Help;
		}
		argindex++;
		
		opto_open(MUX_DEVICE);
		buchse = addsearch(argv[argindex]);	/* Buchsennummer holen */
		ret = muxin(buchse, &wert);
		opto_close();

		if (ret < 0) {
			Tcl_AppendResult(interp, "Timeout while reding MUX", (char *) NULL);
			return TCL_ERROR;
		}
		sprintf(interp->result, "%u 0x%04x %u 0x%04x ", ret, ret, wert, wert);
		return TCL_OK;
	} else {
		Tcl_AppendResult(interp, "bad command \"", argv[1],
						 "\": should be read or set\n", (char *) NULL);
		goto MUX_Help;
	}

  MUX_Help:
	Tcl_AppendResult(interp, "USAGE:  mux set  MUX_CHANNEL mode bit [mode bit]\n",
					 "        mux read MUX_CHANNEL \n\n",
					 "    command: read or set\n",
					 "MUX_CHANNEL: Number of Output-Connector or alias\n",
					 "       mode: c => clear bit [bit ...]\n",
					 "             s => set   bit [bit ...]\n",
					 "             e => set   word\n",
					 "             p => pulse bit [bit ...]\n",
					 "        bit: Value interpreted as Bit-Number 0..15 or 16-Bit-Word",
					 (char *) NULL);
	return TCL_ERROR;
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_MESSMUXCmd --
 *
 *  This procedure is invoked to communicate with the 4x10 Channel
 *      Messmux using the 
 *      This routine is called via the 'avs port' command
 *
 *  See the user documentation for details on what it does.
 *
 * Results:
 *  A standard Tcl result.
 *
 * Side effects:
 *  See the user documentation.
 *
 *----------------------------------------------------------------------
 */

int Tcl_MESSMUXCmd(ClientData dummy, Tcl_Interp * interp, int argc, char **argv)
{
#define MESSMUX_OPTOADDRESS "MESSMUX"
#define MESSMUX_2POINT 0
#define MESSMUX_4POINT 1

	int argindex;
	char argchar;
	
	int i;
	int optoadd = 3, mode = MESSMUX_2POINT, port, buchse, wert = 0, ret=0;
	
	
	/* Optionen parsen und aus dem Argumentfeld entfernen: */

	for (argindex = 1; argindex < argc;) {
		if (argv[argindex][0] == '-') {

			/* Hier die Optionen abfragen: */
			if (strcmp(argv[argindex], "-2point")==0) {
				mode = MESSMUX_2POINT;
			} else if (strcmp(argv[argindex], "-default")==0) {
				mode = MESSMUX_2POINT;
			} else if (strcmp(argv[argindex], "-4point")==0) {
				mode = MESSMUX_4POINT;
			} else {
				Tcl_AppendResult(interp, "ERROR: unknown option '", argv[argindex], "'\n", (char *) NULL);
				return TCL_ERROR;
			}

			/* Zeiger auf Argumente verschieben: */
			if (argindex < argc) {
				for (i = argindex; i < argc; i++)
					argv[i] = argv[i + 1];
			}
			argc--;
		} else
			argindex++;
	}

	if (argc < 2) {
		Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
						 " command \"", (char *) NULL);
		return TCL_ERROR;
	}
	
	argindex = 1;
	argchar = argv[argindex][0];
	
	if ((argchar == 's') && (strcmp(argv[argindex], "set") == 0)) {
		if (argc != 4) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " set ?-options? port connector\"", (char *) NULL);
			return TCL_ERROR;
		}
		argindex ++;
		
		opto_open(MESSMUX_DEVICE);
		optoadd = addsearch(MESSMUX_OPTOADDRESS);	/* MessMUX - Optoaddress holen */
		port = addsearch(argv[argindex]);			/* MessMUX - Portnummer holen */
		buchse = addsearch(argv[argindex + 1]);		/* MessMUX - Buchsennummer holen */
		ret = messmultiplex(port, buchse, mode, optoadd);
		opto_close();

		if (ret < 0) {
			Tcl_AppendResult(interp, "Timeout while setting MESSMUX", (char *) NULL);
			return TCL_ERROR;
		}
		return TCL_OK;
	} else if ((argchar == 'r') && (strcmp(argv[argindex], "read") == 0)) {
		if (argc != 2) {
			Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
							 " read \"", (char *) NULL);
			return TCL_ERROR;
		}
		argindex++;
		
		opto_open(MESSMUX_DEVICE);
		buchse = addsearch(argv[argindex++]);	/* Buchsennummer holen */
		opto_close();

		if (ret < 0) {
			Tcl_AppendResult(interp, "Error while reading MESSMUX-Status", (char *) NULL);
			return TCL_ERROR;
		}
		
		sprintf(interp->result, "%u 0x%04x %u 0x%04x ", ret, ret, wert, wert);
		return TCL_OK;
	} else {
		Tcl_AppendResult(interp, "bad command \"", argv[argindex],
						 "\": should be read or set", (char *) NULL);
		return TCL_ERROR;
	}

}
