#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <tcl.h>
#include <ib.h>

typedef char bool;

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

/*
 * Prototypen
 */

char * Exp_Config_Dir();
	
/*
*  Function name : i3e_input
*
*  Description : liest Daten von einem Geraet in einen Puffer. Die 
*                    Adressierung des Talkerers wird ebenfalls erledigt. 
*  Input :       adf  ist die ascii-Adressierungsfolge der Listener.
*            data  ist die Pufferadresse in dem die zu empfangenen   
*                          Daten gespeichert werden.           
*                    ende  Endekennung, zB CRLF
*  Output :      Anzahl der gelesenen Bytes
*                    -1 zurueckgeliefert wenn ein Sendetimeout auftritt
*                    -2 bei einem Empfangstimeout
*/
int  i3e_input(char *adf, void *data, unsigned ende)
{
	int i3eadr;
	
	i3eadr = ibfind(adf);
	return ibrd(i3eadr, data, 100);	
}
	
/*
*  Function name : i3e_output
*
*  Description : gibt Daten an Geraete aus. Die Adressierung der Listen-
*            er wird ebenfalls erledigt.
*  Input :       adf  ist die ascii-Adressierungsfolge der Listener.
*            data  ist die Pufferadresse in dem die zu sendenden   
*                          Daten gespeichert sind.         
*                    anzahl ist die Zahl der Bytes in data
*  Output :      0 => ok
*                    -1 zurueckgeliefert wenn ein Sendetimeout auftritt
*                    -2 bei einem Empfangstimeout
*/
int  i3e_output(char *adf,void *data,int anzahl)
{
	int i3eadr;
	
	i3eadr = ibfind(adf);
	return ibwrt(i3eadr, data, anzahl);
}

int nwa_read(char *i3eadd, int channel, char *datafilename, char *retmsg)
{
	float *buffer, fdummy;
	char command[100],result[500], match;
	char paramfilename[256];
	
	int numberofpoints, numberofbytes, numberofchars, i;
	FILE  *outputstream, *paramstream;
	const int headersize=4;

	sprintf(command, "CHAN%1d;POIN;OUTPACTI;", channel);
	i3e_output(i3eadd, command, strlen(command));
	i3e_input(i3eadd, result, 0);
	sscanf(result, "%f", &fdummy);
	
	numberofpoints = (int)fdummy;
	numberofbytes = headersize + 2 * numberofpoints * sizeof(float);
	
	if (!(buffer = malloc(numberofbytes+1))) {
		printf("ERROR: Konnte keine %d Bytes fr Daten reservieren\n", numberofbytes);
		return -1;
	}
	
	ibeos(ibfind(i3eadd), 0);
	
	sprintf(command, "FORM5;OUTPDATA;");
	i3e_output(i3eadd, command, strlen(command));
	
	ibrd(ibfind(i3eadd), (void *)buffer, numberofbytes+1);
			
	match = REOS | '\n';
	ibeos(ibfind(i3eadd), match);
	
	/*for (i=0; i<numberofpoints; i++) {
		printf("%d: %g, %g\n", i, buffer[2*i+1], buffer[2*i+2]);
	}*/
	
	printf( "reading %d Bytes\n", numberofbytes-headersize);
	
	outputstream = fopen(datafilename, "w");
	
	if (outputstream != NULL) {
		fwrite((void *)buffer+headersize, 1, numberofbytes-headersize, outputstream);
		fprintf(outputstream, "parameters:\n");
		
		strcpy(paramfilename, Exp_Config_Dir());
		strcat(paramfilename, "/params/");
		sprintf(result, "nwa.s%1d", channel);
		strcat(paramfilename, result);
		paramstream = fopen(paramfilename, "r");
		if (paramstream != NULL) {
			numberofchars = fread((void *)result, 1, 500, paramstream);
			fwrite((void *)result, 1, numberofchars, outputstream);
			fclose(paramstream);
		} else {
			printf("ERROR: no parameter-file \'%s\'\n", paramfilename);
		}
		
		fclose(outputstream);
	} else {
		printf("ERROR: couldn't open output-file \'%s\'\n", datafilename);
	}

	return 0;
}
	
	
int Tcl_NwaCmd(ClientData dummy, Tcl_Interp * interp, int argc, char **argv)
{
	// int fehler = 0;
	char argchar, retmsg[100]="";
	int argindex, arglength, i;

	// bool READ_OLD_LECROY = FALSE;	// Option: -old

	/* Optionen parsen und aus dem Argumentfeld entfernen: */

	for (argindex = 1; argindex < argc;) {
		if (argv[argindex][0] == '-') {

			/* Hier die Optionen abfragen: */
			/*if (strcmp(argv[argindex], "-old") == 0) {
				READ_OLD_LECROY = TRUE;
			} else */{
				Tcl_AppendResult(interp, "ERROR: unknown option '", argv[argindex], "'\n", (char *) NULL);
				return TCL_ERROR;
			}

			/* Zeiger auf Argumente verschieben: */
			if (argindex < argc) {
				for (i = argindex; i < argc; i++)
					argv[i] = argv[i + 1];
			}
			argc--;
		} else
			argindex++;
	}

	if (argc < 3)
		goto NWA_Help;

	/* Argumentindex wieder auf den Beginn der Argumente setzen */
	argindex = 1;

	/* Nun die Argumente parsen: */
	
	if (argc > 1) {
		argchar = argv[argindex][0];
		arglength = strlen(argv[argindex]);
		
		if (argchar == 'r' && strncmp(argv[argindex], "read", arglength) == 0) {
			int nwa_channel;
			char *nwa_name, *filename;
			
			argindex++;

			if (argc - argindex == 3) {
				nwa_name = argv[argindex];
				
				/*if (nwa_address & ERR) {
					Tcl_AppendResult(interp, "ERROR: gpib-Device '", argv[argindex], "' not available\n", (char *) NULL);
					return TCL_ERROR;
				}*/
		
				argindex++;
				if (sscanf(argv[argindex], "%i", &nwa_channel) == 0) {
					Tcl_AppendResult(interp, "ERROR: wrong Channel # '", argv[argindex], "'\n", (char *) NULL);
					return TCL_ERROR;
				}
				
				if (nwa_channel < 1 || nwa_channel > 2) {
					Tcl_AppendResult(interp, "ERROR: Channel #", argv[argindex], " not 1 or 2\n", (char *) NULL);
					return TCL_ERROR;
				}
				
				argindex++;
				
				filename = argv[argindex];
				
			} else {
				Tcl_AppendResult(interp, "ERROR: Wrong number of arguments\n", (char *) NULL);
				return TCL_ERROR;
				
			}
				
				argindex++;		
				
				nwa_read(nwa_name, nwa_channel, filename, retmsg);
				
				Tcl_AppendResult(interp, retmsg, (char *) NULL);
		}
		return TCL_OK;
		
	} else {
NWA_Help:
		Tcl_AppendResult(interp, "USAGE: nwa <cmd> <i3e_label> <chan> <outfile>\n ", (char *) NULL);
		return TCL_ERROR;
	}
}
