/* 
 * tclopto_Bit.c --
 *
 *  This file contains the top-level command routines for the
 *  the Kryo-spezific built-in commands 
 *
 * Copyright (c) 1987-1993 The Regents of the University of California.
 * Copyright (c) 1994-1995 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#define BIT_DEVICE "/dev/opto1"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>
#include <time.h>
#include <ctype.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/times.h>

#include <tcl.h>

#include "optolib.h"
#include "rwadat.h"
#include "tclmod_MUX.h"

#define PKT2       0
#define PKT4       1


static unsigned bitsave=0; /* Variable in der die aktuell gesetzten Bits
							  abgespeichert werden */

/************************************************************************
*																	   	*
* parbit :veraendert Bits an einer Outputbuchse  						*
* Eingabe:	gibt an welche Bits veraendert werden sollen:				*
*					z.B. bits = 0x3 => 0 & 1 werden verandert 2-15 		*
*					bleiben unverndert.								*
*			modus   1 => Bits werden gesetzt							*
*					0 => Bits werden geloescht							*
*					2 => alle bits werden bernommen					*
*					3 => Ausgabe eines Pulses an den angegebenen Bits 	*
*					4 => Bits werden gesetzt und ...-Bits werden 		*
*						 geloescht d.h. es muss eine weitere unsigned   *
*						 Zahl der zu loeschenden Bits angegeben werden	*
* Ergebnis: 0 wenn alles ok, -1 (=Sendetimeout), -2 wenn die MUXoptoadd	*
*			nicht ermittelt werden konnte.								*
*																		*
************************************************************************/
int parbit(unsigned bit, int modus,...)
{
	int optoadd;
	int fehlnr = 0;
	unsigned mbit, nbit, bitc;
	va_list argpoint;

	optoadd = addsearch("RACKBITS");	/* Optoadd von RACKBITS suchen */
	if (optoadd < 0)
		return -2;

	mbit = bitsave;	/* Bits setzen */
	switch (modus) {
	case 0:
		nbit = mbit & (~bit);
		break;
	case 1:
		nbit = mbit | bit;
		break;
	case 3:
		nbit = (mbit & (~bit)) | ((~mbit) & bit);
		break;
	case 4:
		va_start(argpoint, modus);
		bitc = va_arg(argpoint, unsigned);
		va_end(argpoint);
		nbit = mbit | bit;
		nbit = nbit & (~bitc);
		break;
	default:
		nbit = bit;
	};
	if (modus == 3) {			/* kurzen Puls anlegen */
		fehlnr = wordout(optoadd, nbit);
		nbit = mbit;
	} else
		bitsave = nbit;
	
	fehlnr = wordout(optoadd, nbit);

	return fehlnr;
};

int Tcl_OptoBitCmd(ClientData dummy, Tcl_Interp *interp, int argc, char **argv)
{
	int i;
	
	int argindex;
	char argchar;
	
	int modus, ret;
	unsigned bits = 0, bit, bitc = 0;

	if (argc < 2)
		goto Bit_Help;

	argindex = 1;
	argchar = argv[argindex][0];

	opto_open(BIT_DEVICE);
	
	switch (argv[argindex][0]) {	/* Modus ermitteln */
	case 'C':
	case 'c':
		modus = CLR;
		break;
	case 'S':
	case 's':
		modus = SET;
		break;
	case 'P':
	case 'p':
		modus = PULS;
		break;
	case 'E':
	case 'e':
		modus = WRITE;
		break;
	default:
		Tcl_AppendResult(interp, "unknown OptoBit-Command: should be \"", argv[0],
						 " c, s, p or e\"\n", (char *) NULL);
		goto Bit_Help;
	}

	for (i = 2; i < argc; i++) {
		if (argv[i][0] == 'c' || argv[i][0] == 'C') {
			modus = SETCLR;
		}
		if (modus != SETCLR) {
			bit = strtol(argv[i], NULL, 0);		/* Bits holen */
			if (isdigit(argv[i][0]) && (bit < 16) && (modus != WRITE)) {
				bit = 1 << bit;
			}
			bits |= bit;
		} else {
			bit = strtol(argv[i], NULL, 0);		/* Bits holen */
			if (isdigit(argv[i][0]) && (bit < 16) && (modus != WRITE)) {
				bit = 1 << bit;
			}
			bitc |= bit;
		}
	}

	if (modus != SETCLR)
		ret = parbit(bits, modus);
	else
		ret = parbit(bits, modus, bitc);

	if (ret != 0)
		printf("Fehler beim Bit setzen\n");
	
	opto_close();

	if (ret < 0) {
		Tcl_AppendResult(interp, "Timeout while setting Bits", (char *) NULL);
		return TCL_ERROR;
	}
	return TCL_OK;
	
  Bit_Help:
	Tcl_AppendResult(interp, "USAGE:  optobit mode bit [mode bit]\n\n",
					 "       mode: c => clear bit [bit ...]\n",
					 "             s => set   bit [bit ...]\n",
					 "             e => set   word\n",
					 "             p => pulse bit [bit ...]\n",
					 "        bit: Value interpreted as Bit-Number 0..15 or 16-Bit-Word",
					 (char *) NULL);
	return TCL_ERROR;
}

