
/* 
 * tclopto_LeCroy.c --
 *
 *  This file contains the top-level command routines for the
 *  the Kryo-spezific built-in commands 
 *
 * Copyright (c) 1987-1993 The Regents of the University of California.
 * Copyright (c) 1994-1995 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#define PULSER_DEVICE "/dev/opto0"
#define WAITMILLIS 0

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <time.h>
#include <sys/times.h>
#include <ctype.h>

#include "optolib.h"

#include <tcl.h>

#define MIN(a,b)  (((a) < (b)) ? (a) : (b))

/* Prototypen */
int muxaddset(int inadd, int outadd);
char * Exp_Config_Dir();

/********************************************************************************
*																				*																			 																					 
*  	wscalestat:Erzeugt wenn noch nicht vorhanden eine Status und eine Protokoll- *
*			  datei f|r die Pulser.												* 
*																				*																			 																					 
*	so gehts: wpulsstat(pnummer,folge)											*
*	pnummer  = pulsernummer (0-3)												*
*	folge    = Ist i.a. der String der die Pulsfolge beschreibt: 				*
*																				*
*********************************************************************************/
void wscalestat(char scalerchar, int scalefactor)
{
	char stpfad[100] =
	{""};				/* Statusausgabefile, die Kanalnummer wird noch angehaengt */
	char dumstr[100];
	
	FILE *stream;

	strcat(stpfad, Exp_Config_Dir());
	strcat(stpfad, "/params/scaler.s");
	
	sprintf(dumstr, "%c", scalerchar);
	strcat(stpfad, dumstr);		/* Pulsernr an Statusfile anhaengen */

	stream = fopen(stpfad, "w+");	/* Statusfile schreiben oeffnen */
	if (stream != NULL) {
		fprintf(stream, "%d\n", scalefactor);
		fclose(stream);
	};
}

/********************************************************************************
*																				*																			 																					 
*  	wpulsstat:Erzeugt wenn noch nicht vorhanden eine Status und eine Protokoll- *
*			  datei f|r die Pulser.												* 
*																				*																			 																					 
*	so gehts: wpulsstat(pnummer,folge)											*
*	pnummer  = pulsernummer (0-3)												*
*	folge    = Ist i.a. der String der die Pulsfolge beschreibt: 				*
*																				*
*********************************************************************************/
void wpulsstat(int pnummer, char *folge)
{
	char stpfad[100] =
	{""};				/* Statusausgabefile, die Kanalnummer wird noch angehaengt */
	char dumstr[100];
	
	FILE *stream;

	strcat(stpfad, Exp_Config_Dir());
	strcat(stpfad, "/params/pulser.s");
	
	sprintf(dumstr, "%d", pnummer);
	strcat(stpfad, dumstr);		/* Pulsernr an Statusfile anhaengen */

	stream = fopen(stpfad, "w+");	/* Statusfile schreiben oeffnen */
	if (stream != NULL) {
		fprintf(stream, "%s\n", folge);
		fclose(stream);
	};
}

void wrsatstat(char *folge, char *pfad)
{
	FILE *stream;

	stream = fopen(pfad, "w+");	/* Statusfile schreiben oeffnen */
	if (stream != NULL) {
		fprintf(stream, "%s\n", folge);
		fclose(stream);
	};
}


/*
 * Routinen zur Ansteuerung des alten Pulsers
 */

/********************************************************************************
*																				*																			 																					 
*  	pulsprog_old: programmiert einen von vier Pulsern mit einer Pulsfolge			*
*																				*																			 																					 
*	so gehts: pulsprog(nummer,folge,retmsg)										*
*	nummer 	 = pulsernummer (0-3)												*
*	folge    = Ist ein String der die Pulsfolge beschreibt: 					*
*			   z.B. l1000 h500 l20000											*
*			   d.h. 1000 Lowfolgen 5000 Highfolgen und 20000 Lowfolgen			*		
*	retmsg   = Hier wird ein Fehlermeldungsstring zurckgeliefert.				*
*   returnwert= 0 wenn alles ok ist sonst negativ								*
*																				*
*********************************************************************************/
int pulsprog_old(int nummer, char *folge, char *retmsg)
{
	char *tptr, pegel, pfolge[1024];
	long panz = 0l;
	int add = 0, optoadd, fehlnr = 0, entaf = 0, i;
	unsigned lpanz = 0, muxadd;

	strcpy(pfolge, folge);
	optoadd = addsearch("OLDPULSER");	/* Pulseradd suchen */
	if (optoadd > 15) {			/* => Pulser ist ber MUX angeschlossen */
		muxadd = (optoadd / 16 - 1) & 0xf;	/* MUXadd ermitteln */
		optoadd %= 16;			/* Pulseradd errechnen */
		entaf = 1;
		muxaddset(-1, muxadd);	/* Muxoutput neu setzen */
	}
	/* Pulser adressieren */
	fehlnr = wordout(optoadd, 0xE000 | (nummer & 0x3));
	fehlnr = wordout(optoadd, 0x2000);	/* Output = Low */

	for (i = 0; i < strlen(pfolge); i++)	/* alles in Kleinbuchstaben wandeln */
		pfolge[i] = tolower(pfolge[i]);
	tptr = strtok(pfolge, " ");	/* ersten Token suchen */
	while (tptr != NULL) {
		pegel = *tptr++;		/* Pegel merken */
		panz = atol(tptr);		/* Pulszahl ermitteln */
		while (panz != 0l) {
			if ((add++) > 1024) {	/* wenn true => Periodenzahl zu gross */
				fehlnr = -73;
				break;
			};
			if (panz > 40950l) {
				lpanz = (int) MIN(4095l, (panz / 10l));		/* maximal 4095 auf einmal senden */
				panz -= 10l * ((long) lpanz);	/* verbleibende Periodenzahl um lpanz vermindern */
				/* Periodenzahl lpanz senden */
				if (pegel == 'l')
					fehlnr = wordout(optoadd, 0xA000 | lpanz);
				else if (pegel == 'h')
					fehlnr = wordout(optoadd, 0xB000 | lpanz);

			} else {
				lpanz = (int) MIN(4095l, panz);		/* maximal 4095 auf einmal senden */
				panz -= (long) lpanz;	/* verbleibende Periodenzahl um lpanz vermindern */
				/* Periodenzahl lpanz senden */
				if (pegel == 'l')
					fehlnr = wordout(optoadd, 0x8000 | lpanz);
				else if (pegel == 'h')
					fehlnr = wordout(optoadd, 0x9000 | lpanz);
			};
		};
		if (fehlnr < 0) {
			sprintf(retmsg, "Fehler bei Pulsprog\n");
			break;
		};
		tptr = strtok(NULL, " ");	/* naechsten Token suchen */
	};
	wordout(optoadd, 0x6000);	/* Stopbit senden */
	wordout(optoadd, 0x2000);	/* Output = Low */

	if (entaf)
		muxaddset(-1, 14);		/* Muxoutput entadressieren */
	return fehlnr;
};

/********************************************************************************
*																				*																			 																					 
*  	pulsstart_old: startet einen von vier Pulsern.									*
*																				*																			 																					 
*	so gehts: pulsstart(nummer,retmsg)											*
*	nummer 	  = pulsernummer (0-3)												*
*	retmsg   = Hier wird ein Fehlermeldungsstring zurckgeliefert.				*
*   returnwert= 0 wenn alles ok ist sonst negativ								*
*																				*
*********************************************************************************/
int pulsstart_old(int nummer, char *retmsg)
{
	int optoadd, fehlnr = 0;
	unsigned muxadd;

	optoadd = addsearch("OLDPULSER");	/* Pulseradd suchen */
	if (optoadd > 15) {			/* => Pulser ist ber MUX angeschlossen */
		muxadd = (optoadd / 16 - 1) & 0xf;	/* MUXadd ermitteln */
		optoadd %= 16;			/* Pulseradd errechnen */
		muxaddset(-1, muxadd);	/* Muxoutput neu setzen */
	}
	/* Pulser adressieren */
	fehlnr = wordout(optoadd, 0xE000 | (nummer & 0x3));

	fehlnr = wordout(optoadd, 0xC000);	/* Pulser starten */

	if (fehlnr < 0)
		sprintf(retmsg, "Fehler aufgetreten in pulsstart");

	return fehlnr;
}

/*
 * Routinen zur Ansteuerung des neuen Pulsers
 */

/********************************************************************************
*																				*																			 																					 
*  	pulsprog: programmiert einen von vier neuen Pulsern mit einer Pulsfolge	*
*																				*																			 																					 
*	so gehts: pulsprog(nummer,folge,retmsg)									*
*	nummer 	 = pulsernummer (0-3)												*
*	folge    = Ist ein String der die Pulsfolge beschreibt: 					*
*			   z.B. 0v+234 1v-1000 l1000 h500 l20000							*
*			   d.h. 0. Vergleicher gibt bei Takt 234 einen positiven Puls aus,	*
*			   d.h. 1. Vergleicher gibt bei Takt 1000 einen negativen Pils aus. *
*			   Die Pulsfolge ist 1000 Lowfolgen 5000 Highfolgen und 20000 		*
*			   Lowfolgen 														*
*	retmsg   = Hier wird ein Fehlermeldungsstring zurckgeliefert.				*
*   returnwert= 0 wenn alles ok ist sonst negativ								*
*																				*
********************************************************************************/
int pulsprog(int nummer, char *folge, char *retmsg)
{
	char *tptr, pfolge[1024], pegel = 0;
	long panz = 0l;
	int optoadd, fehlnr = 0, entaf = 0, bit, i;
	unsigned muxadd, word = 0, dw;
	long bz = 1;				/* => 1. Bit=0 => l1 kann wegfallen */

	strcpy(pfolge, folge);
	optoadd = addsearch("PULSER");	/* Pulseradd suchen */
	if (optoadd > 15) {			/* => Pulser ist ueber MUX angeschlossen */
		muxadd = (optoadd / 16 - 1) & 0xf;	/* MUXadd ermitteln */
		optoadd %= 16;			/* Pulseradd errechnen */
		entaf = 1;
		muxaddset(-1, muxadd);	/* Muxoutput neu setzen */
	}
	
	fehlnr = wordout(optoadd, 0x0000); /* Einschuebe deselectieren */
	fehlnr = wordout(optoadd, (nummer & 0x3) + 3); /* Pulser adressieren */
	fehlnr = wordout(optoadd, 0x1009);	/* RM=1 PL=0 TM=0 TS=1 */
	fehlnr = wordout(optoadd, 0x3000);  /* 'Start Loading RAM'
										 * RAM auf Datenbus schalten */
	usleep(100);						/* und kurz warten */

	for (i = 0; i < strlen(pfolge); i++)	/* alles in Kleinbuchstaben wandeln */
		pfolge[i] = tolower(pfolge[i]);

	tptr = strtok(pfolge, " ");				/* ersten Token suchen */
	while (tptr != NULL) {
		pegel = *tptr++;					/* Pegel merken */
		if (*tptr == 'v') {					/* => Vergleicher programmieren */
			break;
		}
		
		panz = atol(tptr);					/* Pulszahl ermitteln */
		for (i = 0; i < panz; i++) {
			bit = (int) ((bz) % 16l);		/* Bit ermitteln */
			if (!bit)
				word = 0;					/* word loeschen wenn bit=0 */

			if (pegel == 'h')
				word |= 1 << bit;			/* Bit in word setzen */
			(bz)++;							/* Bitzaehler erhoehen */
			if (bz % 16 == 0) {
				dw = 0x4000 | ((word) & 0xff);
				fehlnr = wordout(optoadd, dw);		/* Lowbyte */
				dw = 0x5000 | (((word) / 256) & 0xff);
				fehlnr = wordout(optoadd, dw);		/* Highbyte */
				fehlnr = wordout(optoadd, 0x6000);	/* Adresse erhoehen */
			}
		}
		
		if (bz >= 16777220l) {	/* Folge ist zu lang */
			puts("Pulsfolge ist zu lang");
			exit(-1);
		}
		
		if (fehlnr < 0) {
			sprintf(retmsg, "Fehler bei Pulsprog\n");
			break;
		}
		
		tptr = strtok(NULL, " ");	/* naechsten Token suchen */
	};
	
	/* die letzten bits schreiben */
	if (bz % 16 != 0) {
		fehlnr = wordout(optoadd, 0x4000 | ((word) & 0xff));	/* Lowbyte */
		fehlnr = wordout(optoadd, 0x5000 | (((word) / 256) & 0xff));	/* Highbyte */
		fehlnr = wordout(optoadd, 0x6000);	/* Adresse erhoehen */
	} else {					/* noch ein wort mit 0 schreiben */
		fehlnr = wordout(optoadd, 0x4000);	/* Lowbyte */
		fehlnr = wordout(optoadd, 0x5000);	/* Highbyte */
		fehlnr = wordout(optoadd, 0x6000);	/* Adresse erhoehen */
	}

	fehlnr = wordout(optoadd, 0x7000);	/* Preset  */
	usleep(100); 						/* kurz warten bis Preset abgeschlossen ist */
					
	/* Endvergleicher setzen */
	fehlnr = wordout(optoadd, 0x2000 + (unsigned) (bz & 0xff));		/* Lowbyte */
	fehlnr = wordout(optoadd, 0x2100 + (unsigned) ((bz >> 8) & 0xff));	/* Highbyte */
	fehlnr = wordout(optoadd, 0x2200 + (unsigned) ((bz >> 16) & 0xff));		/* hh-Byte */

	while (tptr != NULL) {
		i = (int) (pegel) - 0x30;	/* Vergleicher nummer ermitteln */
		tptr++;
		i *= 3;
		panz = atol(tptr) + 1;	/* Ereignisbit ermitteln */
		fehlnr = wordout(optoadd, 0x2000 + (i + 0) * 0x100 + ((unsigned) panz & 0xff));		/* Lowbyte  */
		fehlnr = wordout(optoadd, 0x2000 + (i + 1) * 0x100 + (((unsigned) panz >> 8) & 0xff));	/* Midbyte */
		fehlnr = wordout(optoadd, 0x2000 + (i + 2) * 0x100 + (((unsigned) panz >> 16) & 0xff));		/* Highbyte */
		tptr = strtok(NULL, " ");	/* naechsten Token suchen */
		if (tptr == NULL)
			break;
		pegel = *tptr++;		/* Pegel merken */
		if (*tptr != 'v') {		/* => Vergleicher programmieren */
			break;
		}
	}

	/* Einschuebe deselectieren */
	fehlnr = wordout(optoadd, 0x0000);

	if (entaf)
		muxaddset(-1, 14);		/* Muxoutput emtadressieren */

	return fehlnr;
}

/********************************************************************************
*	pulsteiler : Damit wird ein Vorteiler von des neuen Pulsers gesetzt			*
*																				*
*	nummer ist die Adresse des Vorteilers : 0,1									*
*	teiler ist der Teilerfaktor: 0 = Deaktivieren sonst 1 ..33					*
********************************************************************************/
int pulsteiler(int nummer, int teiler)
{
	int optoadd, fehlnr = 0, entaf = 0;
	unsigned muxadd;

	optoadd = addsearch("PULSER");	/* Pulseradd suchen */
	if (optoadd > 15) {			/* => Pulser ist ber MUX angeschlossen */
		muxadd = (optoadd / 16 - 1) & 0xf;	/* MUXadd ermitteln */
		optoadd %= 16;			/* Pulseradd errechnen */
		entaf = 1;
		muxaddset(-1, muxadd);	/* Muxoutput neu setzen */
	}
	/* alles deselektieren */
	fehlnr = wordout(optoadd, 0x0000);

	/* Vorteiler adressieren */
	fehlnr = wordout(optoadd, 1 + nummer);

	fehlnr = wordout(optoadd, 0x1000 + teiler);		/* Teilerfaktor setzen   */
	fehlnr = wordout(optoadd, 0x0000);

	if (entaf)
		muxaddset(-1, 14);		/* Muxoutput entadressieren */

	return fehlnr;

}
/********************************************************************************
*	pulspreset : Damit wird ein Pulser presetet									*
*																				*
*	nummer ist die Pulsernummer													*
********************************************************************************/
int pulspreset(int nummer)
{
	int optoadd, fehlnr = 0, entaf = 0;
	unsigned muxadd;

	optoadd = addsearch("PULSER");	/* Pulseradd suchen */
	if (optoadd > 15) {			/* => Pulser ist ber MUX angeschlossen */
		muxadd = (optoadd / 16 - 1) & 0xf;	/* MUXadd ermitteln */
		optoadd %= 16;			/* Pulseradd errechnen */
		entaf = 1;
		muxaddset(-1, muxadd);	/* Muxoutput neu setzen */
	}
	fehlnr = wordout(optoadd, 0x0000);	/* Einschuebe deselectieren */
	/* Pulser adressieren */
	fehlnr = wordout(optoadd, (nummer & 0x3) + 3);
	fehlnr = wordout(optoadd, 0x7000);	/* Preset  */
	usleep(100);						/* warten bis Preset abgeschlossen */
	
	fehlnr = wordout(optoadd, 0x0000);	/* Einschuebe deselectieren */

	if (entaf)
		muxaddset(-1, 14);		/* Muxoutput entadressieren */

	return fehlnr;

}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_Pulser --
 *
 *  This procedure is invoked to communicate with the IEEE-Bus-System
 *      using the optolib.c - Functions 
 *  See the user documentation for details on what it does.
 *
 * Results:
 *  A standard Tcl result.
 *
 * Side effects:
 *  See the user documentation.
 *
 *----------------------------------------------------------------------
 */

int Tcl_PulserCmd(ClientData dummy, Tcl_Interp * interp, int argc, char **argv)
{
	size_t arglength;
	int	argindex;
	char argchar=0;
	
	char retmsg[100];
	int pulsernummer, fehlnr=0;
	
	bool USE_OLD_PULSER = FALSE;	/* Option: -old */

	/* Optionen parsen und aus dem Argumentfeld entfernen: */

	for (argindex = 1; argindex < argc;) {
		if (argv[argindex][0] == '-') {
			
			/* Hier die Optionen abfragen: */
			arglength = strlen(argv[argindex]);
			
			if (strncmp(argv[argindex], "-old", arglength)==0) {
				USE_OLD_PULSER = TRUE;
			} else {
				Tcl_AppendResult(interp, "ERROR: unknown option '", argv[argindex], "'\n", (char *) NULL);
				return TCL_ERROR;
			}

			/* Zeiger auf Argumente verschieben: */
			if (argindex < argc) {
				int i;
				for (i = argindex; i < argc; i++)
					argv[i] = argv[i + 1];
			}
			argc--;
		} else
			argindex++;
	}

	/* Argumentindex wieder auf den Beginn der Argumente setzen */
	argindex=1;

	/* Nun die Argumente parsen: */

	if (argc > 1) {
		argchar = argv[argindex][0];
		arglength = strlen(argv[argindex]);

		if (strlen(argv[argindex]) == 1 && isdigit(argv[argindex][0])) {
			goto pulser_program;
			/* Falls erstes Argument eine einstellige Zahl ist ->  Pulser programmieren */
			
/*** pulser program ***/
		} else if (argchar == 'p' && strncmp(argv[argindex], "program", arglength) == 0) {
			char filename[100];
			char pulsfolge[1000];
			argindex++;
			
pulser_program:
			pulsfolge[0]=0;

			if (argc - argindex > 1) {

				if (strlen(argv[argindex]) == 1 && isdigit(argv[argindex][0])) {
					pulsernummer = atoi(argv[argindex]);
					argindex++;

					for (; argindex < argc; argindex++) {
						strcat(pulsfolge, argv[argindex]);
						strcat(pulsfolge, " ");
					}
					
					opto_open(PULSER_DEVICE);
					if (USE_OLD_PULSER) {
						fehlnr = pulsprog_old(pulsernummer, pulsfolge, retmsg);
						sprintf(filename, "%s/params/pulser_old.s%i", Exp_Config_Dir(), pulsernummer);
					} else {
						fehlnr = pulsprog(pulsernummer, pulsfolge, retmsg);
						sprintf(filename, "%s/params/pulser.s%i", Exp_Config_Dir(), pulsernummer);
					}
					opto_close();
					
					if (fehlnr < 0) {
						sprintf(pulsfolge, "%s\n", retmsg);		/* Im Fehlerfall Message ausgeben */
						Tcl_AppendResult(interp, retmsg, (char *) NULL);
					}
					
					wrsatstat(pulsfolge, filename);
					/* wpulsstat(pulsernummer, pulsfolge); */

					return TCL_OK;
				}
			}
			Tcl_AppendResult(interp, " USAGE: \"pulser [flags] <pulser#> l<perioden> h<perioden> ... [<1v10 2v5 ...>]\"\n",
									 "        \"pulser [flags] program <pulser#> l<perioden> h<perioden> ... [<1v10 2v5 ...>]\"\n\n",
							 " pulsernr  = 0-3, ist die Nummer der Pulserkarte \n",
							 " lperioden = die Anzahl der Perioden die der Pulserausgang\n",
							 "             auf Low sein soll.\n",
							 " hperioden = die Anzahl der Perioden die der Pulserausgang\n"
							 "             auf High sein soll.\n",
							 "             Die Summe aller Perioden darf den Wert von 4189185 \n",
							 "             nicht uebersteigen \n",
							 "             Zum Abschluss der Pulsfolge wird der Ausgang automatisch auf\n",
							 "             Low programmiert mit Addresszaehlerreset, d.h. die selbe\n",
							 "             kann ohne Reset sofort wieder gestartet werden\n",
							 " 1v10  bedeutet Vergleicher 1 triggert nach 10 Perioden\n"
							 " 2v5   analog\n", (char *) NULL);
			return TCL_ERROR;

/*** pulser preset ***/
		} else if (argchar == 'p' && strncmp(argv[argindex], "preset", arglength) == 0) {
			argindex++;
			
			if (argc == 3) {
				sscanf(argv[argindex], "%i", &pulsernummer);
				
				if (USE_OLD_PULSER) {
					Tcl_AppendResult(interp, "ERROR: no 'preset' available on OLDPULSER\n", (char *) NULL);
					return TCL_ERROR;
				} else {
					opto_open(PULSER_DEVICE);
					pulspreset(pulsernummer);
					opto_close();
				}
				
				return TCL_OK;
			}
			Tcl_AppendResult(interp, "usage: pulser preset <pulser#>\n", (char *) NULL);
			return TCL_ERROR;

/*** pulser scale ***/
		} else if (argchar == 's' && strncmp(argv[argindex], "scale", arglength) == 0) {
			int teilernummer, teilerwert;
			argindex++;
			
			if (USE_OLD_PULSER) {
				Tcl_AppendResult(interp, "ERROR: option '-old' not possible for 'pulser scale ...'\n", (char *) NULL);
				return TCL_ERROR;
			}
			
			if (argc == 4 && strlen(argv[argindex]) == 1 &&
				strlen(argv[argindex + 1]) <= 2 && isdigit(argv[argindex + 1][0])) {
				if (argv[argindex][0] == 'a')
					teilernummer = 0;
				else if (argv[argindex][0] == 'b')
					teilernummer = 1;
				else {
					Tcl_AppendResult(interp, "<divider#>: a oder b\n", (char *) NULL);
					return TCL_ERROR;
				}
				sscanf(argv[argindex + 1], "%i", &teilerwert);
				if (teilerwert < 0 || teilerwert > 33) {
					Tcl_AppendResult(interp, "<factor> out of Range! [0-33]\n", (char *) NULL);
					return TCL_ERROR;
				}
				opto_open(PULSER_DEVICE);
				pulsteiler(teilernummer, teilerwert);
				wscalestat(argv[argindex][0], teilerwert);
				opto_close();
				return TCL_OK;
			}
			Tcl_AppendResult(interp, "usage: pulser scale <scaler> <factor>\n", (char *) NULL);
			return TCL_ERROR;
			
/*** pulser start ***/
		} else if (argchar == 's' && strncmp(argv[argindex], "start", arglength) == 0) {
			argindex++;
			
			if (argc == 3) {
				sscanf(argv[argindex], "%i", &pulsernummer);
				
				if (USE_OLD_PULSER) {
					opto_open(PULSER_DEVICE);
					printf("start pulser #%i\n", pulsernummer);
					pulsstart_old(pulsernummer, retmsg);
					opto_close();
				} else {
					Tcl_AppendResult(interp, "ERROR: no 'start' available on PULSER\n", (char *) NULL);
					return TCL_ERROR;
				}
				
				if (fehlnr < 0) {
					Tcl_AppendResult(interp, "ERROR: ", retmsg, "\n", (char *) NULL);
					return TCL_ERROR;
				}
				
				return TCL_OK;
			}
			Tcl_AppendResult(interp, "usage: pulser preset <pulser#>\n", (char *) NULL);
			return TCL_ERROR;

/*** unbekannter Befehl: ***/
		} else {
			goto Pulser_Help;
		}
	} else {
Pulser_Help:

		Tcl_AppendResult(interp, "\nusage: pulser [flags] <command> [options]\n\n",
			   					 "  [flags]:    '-old'\t use old rf-pulser\n\n",
							 	 "  <command>:  'preset <pulser#>'            reprogram pulser <pulser#>\n",
								 "              'program <pulser#> <...>' or\n",
								 "              '<pulser#> <...>'             program pulser <pulser#> \n",
							     "              'scale <scaler#> <factor>'    set prescaler <scaler#>\n",
								 "              'start <pulser#>              start old-pulser <pulser#>\n\n"
								 "  use 'pulser <command>' for more info!\n", (char *) NULL);
		return TCL_ERROR;
	}
	return TCL_ERROR;
}
